@@ -39,7 +39,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -66,7 +67,7 @@
 #include <vips/internal.h>
 #include <vips/debug.h>
 
-#include "conversion.h"
+#include "pconversion.h"
 
 typedef struct _VipsJoin {
 	VipsConversion parent_instance;
@@ -117,6 +118,10 @@ vips_join_build( VipsObject *object )
 
 		default:
 			g_assert( 0 );
+
+			/* Keep -Wall happy.
+			 */
+			return( 0 );
 		}
 
 		break;
@@ -139,12 +144,20 @@ vips_join_build( VipsObject *object )
 
 		default:
 			g_assert( 0 );
+
+			/* Keep -Wall happy.
+			 */
+			return( 0 );
 		}
 
 		break;
 
 	default:
 		g_assert( 0 );
+
+		/* Keep -Wall happy.
+		 */
+		return( 0 );
 	}
 
 	if( vips_insert( join->in1, join->in2, &t, x, y,
@@ -174,6 +187,10 @@ vips_join_build( VipsObject *object )
 
 		default:
 			g_assert( 0 );
+
+			/* Keep -Wall happy.
+			 */
+			return( 0 );
 		}
 
 		if( vips_extract_area( t, &t2, 
@@ -200,6 +217,7 @@ vips_join_class_init( VipsJoinClass *class )
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
 	VipsObjectClass *vobject_class = VIPS_OBJECT_CLASS( class );
+	VipsOperationClass *operation_class = VIPS_OPERATION_CLASS( class );
 
 	VIPS_DEBUG_MSG( "vips_join_class_init\n" );
 
@@ -210,6 +228,8 @@ vips_join_class_init( VipsJoinClass *class )
 	vobject_class->description = _( "join a pair of images" );
 	vobject_class->build = vips_join_build;
 
+	operation_class->flags = VIPS_OPERATION_SEQUENTIAL_UNBUFFERED;
+
 	VIPS_ARG_IMAGE( class, "in1", -1, 
 		_( "in1" ), 
 		_( "First input image" ),
