@@ -21,7 +21,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -48,36 +49,81 @@
 #include <vips/internal.h>
 #include <vips/thread.h>
 
-int
-im_tiff2vips( const char *name, IMAGE *out )
+#include "../foreign/tiff.h"
+
+static int
+tiff2vips( const char *name, IMAGE *out, gboolean header_only )
 {
 	char filename[FILENAME_MAX];
 	char mode[FILENAME_MAX];
 	char *p, *q;
 	int page;
-	VipsImage *t;
+	int seq;
 
 	im_filename_split( name, filename, mode );
 
 	page = 0;
+	seq = 0;
 	p = &mode[0];
 	if( (q = im_getnextoption( &p )) ) {
 		page = atoi( q );
 	}
-
-	if( vips_tiffload( filename, &t, 
-		"page", page,
-		NULL ) )
-		return( -1 );
-	if( vips_image_write( t, out ) ) {
-		g_object_unref( t );
-		return( -1 );
+	if( (q = im_getnextoption( &p )) ) {
+		if( im_isprefix( "seq", q ) )
+			seq = 1;
 	}
-	g_object_unref( t );
+
+	/* We need to be compatible with the pre-sequential mode 
+	 * im_tiff2vips(). This returned a "t" if given a "p" image, since it
+	 * used writeline.
+	 *
+	 * If we're writing the image to a "p", switch it to a "t". And only
+	 * for non-tiled (strip) images which we write with writeline.
+	 *
+	 * Don't do this for header read, since we don't want to force a
+	 * malloc if all we are doing is looking at fields.
+	 */
+
+#ifdef HAVE_TIFF
+	if( !header_only &&
+		!seq &&
+		!vips__istifftiled( filename ) &&
+		out->dtype == VIPS_IMAGE_PARTIAL ) {
+		if( vips__image_wio_output( out ) ) 
+			return( -1 );
+	}
+
+	if( header_only ) {
+		if( vips__tiff_read_header( filename, out, page ) )
+			return( -1 );
+	}
+	else {
+		if( vips__tiff_read( filename, out, page, TRUE ) )
+			return( -1 );
+	}
+#else
+	vips_error( "im_tiff2vips", _( "no TIFF support in your libvips" ) ); 
+
+	return( -1 );
+#endif /*HAVE_TIFF*/
 
 	return( 0 );
 }
 
+int
+im_tiff2vips( const char *name, IMAGE *out )
+{
+	return( tiff2vips( name, out, FALSE ) ); 
+}
+
+/* By having a separate header func, we get lazy.c to open via disc/mem.
+ */
+static int
+im_tiff2vips_header( const char *name, IMAGE *out )
+{
+	return( tiff2vips( name, out, TRUE ) ); 
+}
+
 static VipsFormatFlags
 tiff_flags( const char *name )
 {
@@ -115,6 +161,7 @@ vips_format_tiff_class_init( VipsFormatTiffClass *class )
 	object_class->description = _( "TIFF" );
 
 	format_class->is_a = istiff;
+	format_class->header = im_tiff2vips_header;
 	format_class->load = im_tiff2vips;
 	format_class->save = im_vips2tiff;
 	format_class->get_flags = tiff_flags;
