@@ -22,7 +22,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -299,11 +300,12 @@ vips__mat_ismat( const char *filename )
 {
 	mat_t *mat;
 
-	if( !(mat = Mat_Open( filename, MAT_ACC_RDONLY )) )
-		return( 0 );
+	vips_error_freeze();
+	mat = Mat_Open( filename, MAT_ACC_RDONLY );
 	Mat_Close( mat );
+	vips_error_thaw();
 
-	return( 1 );
+	return( mat != NULL );
 }
 
 const char *vips__mat_suffs[] = { ".mat", NULL };
