@@ -2,7 +2,7 @@
  *
  * Benjamin Gilbert
  *
- * Copyright (c) 2011 Carnegie Mellon University
+ * Copyright (c) 2011-2012 Carnegie Mellon University
  *
  * 26/11/11
  *	- initial version
@@ -31,6 +31,14 @@
  * 	- small cleanups
  * 11/4/12
  * 	- fail if both level and associated image are specified
+ * 20/9/12
+ *	- update openslide_open error handling for 3.3.0 semantics
+ *	- switch from deprecated _layer_ functions
+ * 11/10/12
+ * 	- look for tile-width and tile-height properties
+ * 	- use threaded tile cache
+ * 6/8/13
+ * 	- always output solid (not transparent) pixels
  */
 
 /*
@@ -49,7 +57,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -76,14 +85,9 @@
 
 #include "openslide2vips.h"
 
-/* We run our own tile cache. The OpenSlide one can't always keep enough for a
- * complete lines of pixels.
- */
-#define TILE_WIDTH (256)
-#define TILE_HEIGHT (256)
-
 typedef struct {
 	openslide_t *osr;
+
 	char *associated;
 
 	/* Only valid if associated == NULL.
@@ -91,26 +95,40 @@ typedef struct {
 	int32_t level;
 	double downsample;
 	uint32_t bg;
+
+	/* Try to get these from openslide properties.
+	 */
+	int tile_width;
+	int tile_height;
 } ReadSlide;
 
 int
 vips__openslide_isslide( const char *filename )
 {
 	openslide_t *osr;
-	const char *vendor;
 	int ok;
 
 	ok = 0;
-	if( (osr = openslide_open( filename )) ) {
+	vips_error_freeze();
+	osr = openslide_open( filename );
+	vips_error_thaw();
+
+	if( osr ) {
+		const char *vendor;
+
 		/* Generic tiled tiff images can be opened by openslide as
 		 * well. Only offer to load this file if it's not a generic
 		 * tiff since we want vips_tiffload() to handle these.
 		 */
 		vendor = openslide_get_property_value( osr,
 			OPENSLIDE_PROPERTY_NAME_VENDOR );
+
+		/* vendor will be NULL if osr is in error state.
+		 */
 		if( vendor &&
 			strcmp( vendor, "generic-tiff" ) != 0 )
 			ok = 1;
+
 		openslide_close( osr );
 	} 
 
@@ -148,6 +166,7 @@ readslide_new( const char *filename, VipsImage *out,
 {
 	ReadSlide *rslide;
 	int64_t w, h;
+	const char *error;
 	const char *background;
 	const char * const *properties;
 
@@ -166,15 +185,27 @@ readslide_new( const char *filename, VipsImage *out,
 	rslide->level = level;
 	rslide->associated = g_strdup( associated );
 
+	/* Non-crazy defaults, override below if we can.
+	 */
+	rslide->tile_width = 256;
+	rslide->tile_height = 256;
+
 	rslide->osr = openslide_open( filename );
 	if( rslide->osr == NULL ) {
 		vips_error( "openslide2vips", 
-			"%s", _( "failure opening slide" ) );
+			"%s", _( "unsupported slide format" ) );
+		return( NULL );
+	}
+
+	error = openslide_get_error( rslide->osr );
+	if( error ) {
+		vips_error( "openslide2vips",
+			_( "opening slide: %s" ), error );
 		return( NULL );
 	}
 
 	if( level < 0 || 
-		level >= openslide_get_layer_count( rslide->osr ) ) {
+		level >= openslide_get_level_count( rslide->osr ) ) {
 		vips_error( "openslide2vips",
 			"%s", _( "invalid slide level" ) );
 		return( NULL );
@@ -192,12 +223,29 @@ readslide_new( const char *filename, VipsImage *out,
 		vips_demand_hint( out, VIPS_DEMAND_STYLE_THINSTRIP, NULL );
 	} 
 	else {
-		openslide_get_layer_dimensions( rslide->osr, 
+		char buf[256];
+		const char *value;
+
+		openslide_get_level_dimensions( rslide->osr,
 			level, &w, &h );
-		rslide->downsample = openslide_get_layer_downsample(
+		rslide->downsample = openslide_get_level_downsample(
 			rslide->osr, level );
 		vips_image_set_int( out, "slide-level", level );
 		vips_demand_hint( out, VIPS_DEMAND_STYLE_SMALLTILE, NULL );
+
+		/* Try to get tile width/height. An undocumented, experimental
+		 * feature.
+		 */
+		vips_snprintf( buf, 256, 
+			"openslide.level[%d].tile-width", level );
+		if( (value = openslide_get_property_value( rslide->osr, buf )) )
+			rslide->tile_width = atoi( value );
+		vips_snprintf( buf, 256, 
+			"openslide.level[%d].tile-height", level );
+		if( (value = openslide_get_property_value( rslide->osr, buf )) )
+			rslide->tile_height = atoi( value );
+		if( value )
+			VIPS_DEBUG_MSG( "readslide_new: found tile-size\n" );
 	}
 
 	rslide->bg = 0xffffff;
@@ -217,6 +265,7 @@ readslide_new( const char *filename, VipsImage *out,
 		return( NULL );
 	}
 
+
 	vips_image_init_fields( out, w, h, 4, VIPS_FORMAT_UCHAR,
 		VIPS_CODING_NONE, VIPS_INTERPRETATION_RGB, 1.0, 1.0 );
 
@@ -237,9 +286,7 @@ int
 vips__openslide_read_header( const char *filename, VipsImage *out, 
 	int level, char *associated )
 {
-	ReadSlide *rslide;
-
-	if( !(rslide = readslide_new( filename, out, level, associated )) )
+	if( !readslide_new( filename, out, level, associated ) )
 		return( -1 );
 
 	return( 0 );
@@ -247,7 +294,7 @@ vips__openslide_read_header( const char *filename, VipsImage *out,
 
 static int
 vips__openslide_generate( VipsRegion *out, 
-	void *seq, void *_rslide, void *unused, gboolean *stop )
+	void *_seq, void *_rslide, void *unused, gboolean *stop )
 {
 	ReadSlide *rslide = _rslide;
 	uint32_t bg = rslide->bg;
@@ -261,13 +308,18 @@ vips__openslide_generate( VipsRegion *out,
 	VIPS_DEBUG_MSG( "vips__openslide_generate: %dx%d @ %dx%d\n",
 		r->width, r->height, r->left, r->top );
 
-	/* We're inside a cache, so requests should always be TILE_WIDTH by
-	 * TILE_HEIGHT pixels and on a tile boundary.
+	/* We're inside a cache, so requests should always be
+	 * tile_width by tile_height pixels and on a tile boundary.
 	 */
-	g_assert( (r->left % TILE_WIDTH) == 0 );
-	g_assert( (r->height % TILE_HEIGHT) == 0 );
-	g_assert( r->width <= TILE_WIDTH );
-	g_assert( r->height <= TILE_HEIGHT );
+	g_assert( (r->left % rslide->tile_width) == 0 );
+	g_assert( (r->top % rslide->tile_height) == 0 );
+	g_assert( r->width <= rslide->tile_width );
+	g_assert( r->height <= rslide->tile_height );
+
+	/* The memory on the region should be contiguous for our ARGB->RGBA
+	 * loop below.
+	 */
+	g_assert( VIPS_REGION_LSKIP( out ) == r->width * 4 );
 
 	openslide_read_region( rslide->osr, 
 		buf,
@@ -284,18 +336,29 @@ vips__openslide_generate( VipsRegion *out,
 		return( -1 );
 	}
 
-	/* Convert from ARGB to RGBA and undo premultiplication.
+	/* Convert from ARGB to RGBA and undo premultiplication. Since we are
+	 * inside a cache, we know buf must be continuous.
+	 *
+	 * We throw away transparency. Formats like Mirax use transparent + bg
+	 * colour for areas with no useful pixels. But if we output
+	 * transparent pixels and then convert to RGB for jpeg write later, we
+	 * would have to pass the bg colour down the pipe somehow. The
+	 * structure of dzsave makes this tricky.
+	 *
+	 * We could output plain RGB instead, but that would break
+	 * compatibility with older vipses.
 	 */
 	for( i = 0; i < n; i++ ) {
-		uint32_t x = buf[i];
+		uint32_t *p = buf + i;
+		uint32_t x = *p;
 		uint8_t a = x >> 24;
-		VipsPel *out = (VipsPel *) (buf + i);
+		VipsPel *out = (VipsPel *) p;
 
 		if( a != 0 ) {
 			out[0] = 255 * ((x >> 16) & 255) / a;
 			out[1] = 255 * ((x >> 8) & 255) / a;
 			out[2] = 255 * (x & 255) / a;
-			out[3] = a;
+			out[3] = 255;
 		} 
 		else {
 			/* Use background color.
@@ -303,7 +366,7 @@ vips__openslide_generate( VipsRegion *out,
 			out[0] = (bg >> 16) & 255;
 			out[1] = (bg >> 8) & 255;
 			out[2] = bg & 255;
-			out[3] = 0;
+			out[3] = 255;
 		}
 	}
 
@@ -320,10 +383,6 @@ vips__openslide_read( const char *filename, VipsImage *out, int level )
 	VIPS_DEBUG_MSG( "vips__openslide_read: %s %d\n", 
 		filename, level );
 
-	/* Tile cache: keep enough for two complete rows of tiles. OpenSlide
-	 * has its own tile cache, but it's not large enough for a complete
-	 * scan line.
-	 */
 	raw = vips_image_new();
 	vips_object_local( out, raw );
 
@@ -338,9 +397,11 @@ vips__openslide_read( const char *filename, VipsImage *out, int level )
 	 * 50%.
 	 */
 	if( vips_tilecache( raw, &t, 
-		"tile_width", TILE_WIDTH, 
-		"tile_height", TILE_WIDTH,
-		"max_tiles", (int) (1.5 * (1 + raw->Xsize / TILE_WIDTH)),
+		"tile_width", rslide->tile_width, 
+		"tile_height", rslide->tile_height,
+		"max_tiles", 
+			(int) (1.5 * (1 + raw->Xsize / rslide->tile_width)),
+		"threaded", TRUE,
 		NULL ) ) 
 		return( -1 );
 	if( vips_image_write( t, out ) ) {
