@@ -22,7 +22,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -87,7 +88,7 @@ vips_foreign_save_tiff_build( VipsObject *object )
 	VipsForeignSave *save = (VipsForeignSave *) object;
 	VipsForeignSaveTiff *tiff = (VipsForeignSaveTiff *) object;
 
-	char *p;
+	const char *p;
 
 	if( VIPS_OBJECT_CLASS( vips_foreign_save_tiff_parent_class )->
 		build( object ) )
@@ -95,14 +96,14 @@ vips_foreign_save_tiff_build( VipsObject *object )
 
 	/* Default xres/yres to the values from the image.
 	 */
-	if( !vips_object_get_argument_assigned( object, "xres" ) )
+	if( !vips_object_argument_isset( object, "xres" ) )
 		tiff->xres = save->ready->Xres * 10.0;
-	if( !vips_object_get_argument_assigned( object, "yres" ) )
+	if( !vips_object_argument_isset( object, "yres" ) )
 		tiff->yres = save->ready->Yres * 10.0;
 
 	/* resunit param overrides resunit metadata.
 	 */
-	if( !vips_object_get_argument_assigned( object, "resunit" ) &&
+	if( !vips_object_argument_isset( object, "resunit" ) &&
 		vips_image_get_typeof( save->ready, 
 			VIPS_META_RESOLUTION_UNIT ) &&
 		!vips_image_get_string( save->ready, 
@@ -128,24 +129,6 @@ vips_foreign_save_tiff_build( VipsObject *object )
 	return( 0 );
 }
 
-/* Save a bit of typing.
- */
-#define UC VIPS_FORMAT_UCHAR
-#define C VIPS_FORMAT_CHAR
-#define US VIPS_FORMAT_USHORT
-#define S VIPS_FORMAT_SHORT
-#define UI VIPS_FORMAT_UINT
-#define I VIPS_FORMAT_INT
-#define F VIPS_FORMAT_FLOAT
-#define X VIPS_FORMAT_COMPLEX
-#define D VIPS_FORMAT_DOUBLE
-#define DX VIPS_FORMAT_DPCOMPLEX
-
-static int bandfmt_tiff[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   UC, UC, US, S,  US, US, F,  F,  F,  F
-};
-
 static void
 vips_foreign_save_tiff_class_init( VipsForeignSaveTiffClass *class )
 {
@@ -164,7 +147,6 @@ vips_foreign_save_tiff_class_init( VipsForeignSaveTiffClass *class )
 	foreign_class->suffs = vips__foreign_tiff_suffs;
 
 	save_class->saveable = VIPS_SAVEABLE_ANY;
-	save_class->format_table = bandfmt_tiff;
 	save_class->coding[VIPS_CODING_LABQ] = TRUE;
 
 	VIPS_ARG_STRING( class, "filename", 1, 
