@@ -126,6 +126,16 @@
  * 2/12/11
  * 	- make into a simple function call ready to be wrapped as a new-style
  * 	  VipsForeign class
+ * 21/3/12
+ * 	- bump max layer buffer up
+ * 2/6/12
+ * 	- copy jpeg pyramid in gather in RGB mode ... tiff4 doesn't do ycbcr
+ * 	  mode
+ * 7/8/12
+ * 	- be more cautious enabling YCbCr mode
+ * 24/9/13
+ * 	- support many more vips formats, eg. complex, 32-bit int, any number
+ * 	  of bands, etc., see the tiff loader
  */
 
 /*
@@ -144,7 +154,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -155,6 +166,7 @@
  */
 
 /* 
+#define DEBUG_VERBOSE
 #define DEBUG
  */
 
@@ -182,7 +194,7 @@
 /* Max no of tiles we buffer in a layer. Enough to buffer a line of 64x64
  * tiles on a 100k pixel across image.
  */
-#define MAX_LAYER_BUFFER (1000)
+#define MAX_LAYER_BUFFER (10000)
 
 /* Bits we OR together for quadrants in a tile.
  */
@@ -443,6 +455,7 @@ static int
 write_tiff_header( TiffWrite *tw, TIFF *tif, int width, int height )
 {
 	uint16 v[1];
+	int format; 
 
 	/* Output base header fields.
 	 */
@@ -452,16 +465,10 @@ write_tiff_header( TiffWrite *tw, TIFF *tif, int width, int height )
 	TIFFSetField( tif, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT );
 	TIFFSetField( tif, TIFFTAG_COMPRESSION, tw->compression );
 
-	if( tw->compression == COMPRESSION_JPEG ) {
+	if( tw->compression == COMPRESSION_JPEG ) 
 		TIFFSetField( tif, TIFFTAG_JPEGQUALITY, tw->jpqual );
 
-		/* Enable rgb->ycbcr conversion in the jpeg write. See also
-		 * the photometric selection below.
-		 */
-		TIFFSetField( tif, TIFFTAG_JPEGCOLORMODE, JPEGCOLORMODE_RGB );
-	}
-
-	if( tw->predictor != -1 ) 
+	if( tw->predictor != VIPS_FOREIGN_TIFF_PREDICTOR_NONE ) 
 		TIFFSetField( tif, TIFFTAG_PREDICTOR, tw->predictor );
 
 	/* Don't write mad resolutions (eg. zero), it confuses some programs.
@@ -509,6 +516,9 @@ write_tiff_header( TiffWrite *tw, TIFF *tif, int width, int height )
 
 		case 3:
 		case 4:
+			/* could be: RGB, RGBA, CMYK, LAB, LABA, generic
+			 * multi-band image.
+			 */
 			if( tw->im->Type == VIPS_INTERPRETATION_LAB || 
 				tw->im->Type == VIPS_INTERPRETATION_LABS ) 
 				photometric = PHOTOMETRIC_CIELAB;
@@ -518,12 +528,16 @@ write_tiff_header( TiffWrite *tw, TIFF *tif, int width, int height )
 					TIFFTAG_INKSET, INKSET_CMYK );
 			}
 			else if( tw->compression == COMPRESSION_JPEG &&
-				tw->im->Bands == 3 ) 
+				tw->im->Bands == 3 &&
+				tw->im->BandFmt == VIPS_FORMAT_UCHAR ) { 
 				/* This signals to libjpeg that it can do
 				 * YCbCr chrominance subsampling from RGB, not
 				 * that we will supply the image as YCbCr.
 				 */
 				photometric = PHOTOMETRIC_YCBCR;
+				TIFFSetField( tif, TIFFTAG_JPEGCOLORMODE, 
+					JPEGCOLORMODE_RGB );
+			}
 			else
 				photometric = PHOTOMETRIC_RGB;
 
@@ -532,18 +546,23 @@ write_tiff_header( TiffWrite *tw, TIFF *tif, int width, int height )
 				v[0] = EXTRASAMPLE_ASSOCALPHA;
 				TIFFSetField( tif, TIFFTAG_EXTRASAMPLES, 1, v );
 			}
+
 			break;
 
 		case 5:
-			if( tw->im->Type == VIPS_INTERPRETATION_CMYK ) {
-				photometric = PHOTOMETRIC_SEPARATED;
-				TIFFSetField( tif, 
-					TIFFTAG_INKSET, INKSET_CMYK );
-			}
+			/* Only CMYKA
+			 */
+			photometric = PHOTOMETRIC_SEPARATED;
+			TIFFSetField( tif, TIFFTAG_INKSET, INKSET_CMYK );
+			v[0] = EXTRASAMPLE_ASSOCALPHA;
+			TIFFSetField( tif, TIFFTAG_EXTRASAMPLES, 1, v );
 			break;
 
 		default:
-			g_assert( 0 );
+			/* Who knows. Just call it RGB.
+			 */
+			photometric = PHOTOMETRIC_RGB;
+			break; 
 		}
 
 		TIFFSetField( tif, TIFFTAG_PHOTOMETRIC, photometric );
@@ -563,10 +582,19 @@ write_tiff_header( TiffWrite *tw, TIFF *tif, int width, int height )
 		TIFFSetField( tif, TIFFTAG_SUBFILETYPE, FILETYPE_REDUCEDIMAGE );
 	}
 
-	/* Sample format ... for float, we write IEEE.
+	/* Sample format.
 	 */
-	if( tw->im->BandFmt == VIPS_FORMAT_FLOAT )
-		TIFFSetField( tif, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_IEEEFP );
+	format = SAMPLEFORMAT_UINT;
+	if( vips_band_format_isuint( tw->im->BandFmt ) )
+		format = SAMPLEFORMAT_UINT;
+	else if( vips_band_format_isint( tw->im->BandFmt ) )
+		format = SAMPLEFORMAT_INT;
+	else if( vips_band_format_isfloat( tw->im->BandFmt ) )
+		format = SAMPLEFORMAT_IEEEFP;
+	else if( vips_band_format_iscomplex( tw->im->BandFmt ) )
+		format = SAMPLEFORMAT_COMPLEXIEEEFP;
+
+	TIFFSetField( tif, TIFFTAG_SAMPLEFORMAT, format );
 
 	return( 0 );
 }
@@ -669,7 +697,7 @@ find_new_tile( PyramidLayer *layer )
 {
 	int i;
 
-	/* Exisiting buffer we have finished with? 
+	/* Existing buffer we have finished with? 
 	 */
 	for( i = 0; i < MAX_LAYER_BUFFER; i++ )
 		if( layer->tiles[i].bits == PYR_ALL ) 
@@ -682,7 +710,7 @@ find_new_tile( PyramidLayer *layer )
 			if( !(layer->tiles[i].tile = 
 				vips_region_new( layer->tw->im )) )
 				return( -1 );
-			vips__region_no_ownership( layer->tiles[i].tile );
+
 			return( i );
 		}
 
@@ -721,6 +749,7 @@ find_tile( PyramidLayer *layer, VipsRect *pos )
 		return( -1 );
 	if( vips_region_buffer( layer->tiles[i].tile, pos ) )
 		return( -1 );
+	vips__region_no_ownership( layer->tiles[i].tile );
 	layer->tiles[i].bits = PYR_NONE;
 
 	/* Do any quadrants of this tile fall entirely outside the image? 
@@ -904,17 +933,18 @@ shrink_region( VipsRegion *from, VipsRect *area,
 /* Write a tile from a layer.
  */
 static int
-save_tile( TiffWrite *tw, TIFF *tif, VipsPel *tbuf, VipsRegion *reg, VipsRect *area )
+save_tile( TiffWrite *tw, 
+	TIFF *tif, VipsPel *tbuf, VipsRegion *reg, VipsRect *area )
 {
 	/* Have to repack pixels.
 	 */
 	pack2tiff( tw, reg, tbuf, area );
 
-#ifdef DEBUG
+#ifdef DEBUG_VERBOSE
 	printf( "Writing %dx%d pixels at position %dx%d to image %s\n",
 		tw->tilew, tw->tileh, area->left, area->top,
 		TIFFFileName( tif ) );
-#endif /*DEBUG*/
+#endif /*DEBUG_VERBOSE*/
 
 	/* Write to TIFF! easy.
 	 */
@@ -1056,12 +1086,28 @@ write_tif_tilewise( TiffWrite *tw )
 {
 	VipsImage *im = tw->im;
 
+	/* Double check: buffers should match in size, except for onebit and
+	 * labq modes.  
+	 */
+{
+	size_t vips_tile_size = 
+		VIPS_IMAGE_SIZEOF_PEL( im ) * tw->tilew * tw->tileh; 
+
+	if( tw->im->Coding != VIPS_CODING_LABQ &&
+		!tw->onebit &&
+		TIFFTileSize( tw->tif ) != vips_tile_size ) { 
+		vips_error( "vips2tiff", 
+			"%s", _( "unsupported image format" ) );
+		return( -1 );
+	}
+}
+
 	g_assert( !tw->tbuf );
 	if( !(tw->tbuf = vips_malloc( NULL, TIFFTileSize( tw->tif ) )) ) 
 		return( -1 );
 
 	g_assert( !tw->write_lock );
-	tw->write_lock = g_mutex_new();
+	tw->write_lock = vips_g_mutex_new();
 
 	/* Write pyramid too? Only bother if bigger than tile size.
 	 */
@@ -1118,6 +1164,18 @@ write_tif_stripwise( TiffWrite *tw )
 {
 	g_assert( !tw->tbuf );
 
+	/* Double check: buffers should match in size, except for onebit and
+	 * labq modes.  
+	 */
+	if( tw->im->Coding != VIPS_CODING_LABQ &&
+		!tw->onebit &&
+		TIFFScanlineSize( tw->tif ) != 
+			VIPS_IMAGE_SIZEOF_LINE( tw->im ) ) { 
+		vips_error( "vips2tiff", 
+			"%s", _( "unsupported image format" ) );
+		return( -1 );
+	}
+
 	if( !(tw->tbuf = vips_malloc( NULL, TIFFScanlineSize( tw->tif ) )) ) 
 		return( -1 );
 
@@ -1135,13 +1193,23 @@ delete_files( TiffWrite *tw )
 	PyramidLayer *layer = tw->layer;
 
 	if( tw->bname ) {
+#ifndef DEBUG
 		unlink( tw->bname );
+#else
+		printf( "delete_files: leaving %s\n", tw->bname );
+#endif /*DEBUG*/
+
 		tw->bname = NULL;
 	}
 
 	for( layer = tw->layer; layer; layer = layer->below ) 
 		if( layer->lname ) {
+#ifndef DEBUG
 			unlink( layer->lname );
+#else
+			printf( "delete_files: leaving %s\n", layer->lname );
+#endif /*DEBUG*/
+
 			layer->lname = NULL;
 		}
 }
@@ -1151,13 +1219,11 @@ delete_files( TiffWrite *tw )
 static void
 free_tiff_write( TiffWrite *tw )
 {
-#ifndef DEBUG
 	delete_files( tw );
-#endif /*DEBUG*/
 
 	VIPS_FREEF( TIFFClose, tw->tif );
 	VIPS_FREEF( vips_free, tw->tbuf );
-	VIPS_FREEF( g_mutex_free, tw->write_lock );
+	VIPS_FREEF( vips_g_mutex_free, tw->write_lock );
 	VIPS_FREEF( free_pyramid, tw->layer );
 	VIPS_FREEF( vips_free, tw->icc_profile );
 }
@@ -1190,6 +1256,10 @@ get_compression( VipsForeignTiffCompression compression )
 	default:
 		g_assert( 0 );
 	}
+
+	/* Keep -Wall happy.
+	 */
+	return( -1 );
 }
 
 static int
@@ -1204,6 +1274,10 @@ get_resunit( VipsForeignTiffResunit resunit )
 	default:
 		g_assert( 0 );
 	}
+
+	/* Keep -Wall happy.
+	 */
+	return( -1 );
 }
 
 /* Make and init a TiffWrite.
@@ -1273,14 +1347,18 @@ make_tiff_write( VipsImage *im, const char *filename,
 
 	/* Only 1-bit-ize 8 bit mono images.
 	 */
-	if( tw->onebit ) {
-		if( im->Coding != VIPS_CODING_NONE || 
+	if( tw->onebit &&
+		(im->Coding != VIPS_CODING_NONE || 
 			im->BandFmt != VIPS_FORMAT_UCHAR ||
-			im->Bands != 1 ) 
-			tw->onebit = 0;
+			im->Bands != 1) ) {
+		vips_warn( "vips2tiff", 
+			"%s", _( "can only squash 1 band uchar images -- "
+				"disabling squash" ) );
+		tw->onebit = 0;
 	}
 
-	if( tw->onebit && tw->compression == COMPRESSION_JPEG ) {
+	if( tw->onebit && 
+		tw->compression == COMPRESSION_JPEG ) {
 		vips_warn( "vips2tiff", 
 			"%s", _( "can't have 1-bit JPEG -- disabling JPEG" ) );
 		tw->compression = COMPRESSION_NONE;
@@ -1334,15 +1412,33 @@ tiff_copy( TiffWrite *tw, TIFF *out, TIFF *in )
 	CopyField( TIFFTAG_ROWSPERSTRIP, i32 );
 	CopyField( TIFFTAG_SUBFILETYPE, i32 );
 
-	if( tw->predictor != -1 ) 
+	if( tw->predictor != VIPS_FOREIGN_TIFF_PREDICTOR_NONE ) 
 		TIFFSetField( out, TIFFTAG_PREDICTOR, tw->predictor );
 
 	/* TIFFTAG_JPEGQUALITY is a pesudo-tag, so we can't copy it.
 	 * Set explicitly from TiffWrite.
 	 */
-	if( tw->compression == COMPRESSION_JPEG ) 
+	if( tw->compression == COMPRESSION_JPEG ) {
 		TIFFSetField( out, TIFFTAG_JPEGQUALITY, tw->jpqual );
 
+		/* Only for three-band, 8-bit images.
+		 */
+		if( tw->im->Bands == 3 &&
+			tw->im->BandFmt == VIPS_FORMAT_UCHAR ) { 
+			/* Enable rgb->ycbcr conversion in the jpeg write. 
+			 */
+			TIFFSetField( out, 
+				TIFFTAG_JPEGCOLORMODE, JPEGCOLORMODE_RGB );
+
+			/* And we want ycbcr expanded to rgb on read. Otherwise
+			 * TIFFTileSize() will give us the size of a chrominance
+			 * subsampled tile.
+			 */
+			TIFFSetField( in, 
+				TIFFTAG_JPEGCOLORMODE, JPEGCOLORMODE_RGB );
+		}
+	}
+
 	/* We can't copy profiles :( Set again from TiffWrite.
 	 */
 	if( embed_profile( tw, out ) )
@@ -1357,7 +1453,7 @@ tiff_copy( TiffWrite *tw, TIFF *out, TIFF *in )
 		 * here to save compression/decompression, but sadly it seems
 		 * not to work :-( investigate at some point.
 		 */
-		len = TIFFReadEncodedTile( in, tile, buf, (tsize_t) -1 );
+		len = TIFFReadEncodedTile( in, tile, buf, -1 );
 		if( len < 0 ||
 			TIFFWriteEncodedTile( out, tile, buf, len ) < 0 ) {
 			vips_free( buf );
@@ -1426,7 +1522,6 @@ gather_pyramid( TiffWrite *tw )
 	return( 0 );
 }
 
-
 int 
 vips__tiff_write( VipsImage *in, const char *filename, 
 	VipsForeignTiffCompression compression, int Q, 
@@ -1445,32 +1540,12 @@ vips__tiff_write( VipsImage *in, const char *filename,
 	printf( "tiff2vips: libtiff version is \"%s\"\n", TIFFGetVersion() );
 #endif /*DEBUG*/
 
-	/* Override the default TIFF error handler.
-	 */
-	TIFFSetErrorHandler( vips__thandler_error );
-	TIFFSetWarningHandler( vips__thandler_warning );
+	vips__tiff_init();
 
 	/* Check input image.
 	 */
 	if( vips_check_coding_known( "vips2tiff", in ) )
 		return( -1 );
-	if( in->BandFmt != VIPS_FORMAT_UCHAR && 
-		!(in->BandFmt == VIPS_FORMAT_SHORT && 
-			in->Type == VIPS_INTERPRETATION_LABS) &&
-		in->BandFmt != VIPS_FORMAT_USHORT &&
-		in->BandFmt != VIPS_FORMAT_FLOAT ) {
-		vips_error( "vips2tiff", "%s", 
-			_( "unsigned 8-bit int, 16-bit int, "
-			"and 32-bit float only" ) );
-		return( -1 );
-	}
-	if( in->Coding == VIPS_CODING_NONE ) {
-		if( in->Bands < 1 || in->Bands > 5 ) {
-			vips_error( "vips2tiff", 
-				"%s", _( "1 to 5 bands only" ) );
-			return( -1 );
-		}
-	}
 
 	/* Make output image. If this is a pyramid, write the base image to
 	 * tmp/xx.tif rather than fred.tif.
@@ -1525,7 +1600,8 @@ vips__tiff_write( VipsImage *in, const char *filename,
 
 	/* Gather layers together into final pyramid file.
 	 */
-	if( tw->pyramid && gather_pyramid( tw ) ) {
+	if( tw->pyramid && 
+		gather_pyramid( tw ) ) {
 		free_tiff_write( tw );
 		return( -1 );
 	}
