@@ -40,6 +40,14 @@
  * 	- add a longjmp() to our error handler to stop the default one running
  * 13/3/12
  * 	- add ICC profile read/write
+ * 15/3/12
+ * 	- better alpha handling
+ * 	- sanity check pixel geometry before allowing read
+ * 17/6/12
+ * 	- more alpha fixes ... some images have no transparency chunk but
+ * 	  still set color_type to alpha
+ * 16/7/13
+ * 	- more robust error handling from libpng
  */
 
 /*
@@ -58,7 +66,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -81,6 +90,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <vips/vips.h>
 #include <vips/internal.h>
@@ -97,9 +107,9 @@
 static void
 user_error_function( png_structp png_ptr, png_const_charp error_msg )
 {
-	vips_error( "png2vips", "%s", error_msg );
+	vips_error( "vipspng", "%s", error_msg );
 
-	/* This function must not return, or the default error handler will be
+	/* This function must not return or the default error handler will be
 	 * invoked.
 	 */
 	longjmp( png_jmpbuf( png_ptr ), -1 ); 
@@ -108,7 +118,7 @@ user_error_function( png_structp png_ptr, png_const_charp error_msg )
 static void
 user_warning_function( png_structp png_ptr, png_const_charp warning_msg )
 {
-	vips_error( "png2vips", "%s", warning_msg );
+	vips_error( "vipspng", "%s", warning_msg );
 }
 
 /* What we track during a PNG read.
@@ -116,11 +126,23 @@ user_warning_function( png_structp png_ptr, png_const_charp warning_msg )
 typedef struct {
 	char *name;
 	VipsImage *out;
+	gboolean readbehind; 
 
-	FILE *fp;
+	int y_pos;
 	png_structp pPng;
 	png_infop pInfo;
 	png_bytep *row_pointer;
+
+	/* For FILE input.
+	 */
+	FILE *fp;
+
+	/* For memory input.
+	 */
+	char *buffer;
+	size_t length;
+	size_t read_pos;
+
 } Read;
 
 static void
@@ -133,26 +155,28 @@ read_destroy( VipsImage *out, Read *read )
 }
 
 static Read *
-read_new( const char *name, VipsImage *out )
+read_new( VipsImage *out, gboolean readbehind )
 {
 	Read *read;
 
 	if( !(read = VIPS_NEW( out, Read )) )
 		return( NULL );
 
-	read->name = vips_strdup( VIPS_OBJECT( out ), name );
+	read->name = NULL;
+	read->readbehind = readbehind;
 	read->out = out;
-	read->fp = NULL;
+	read->y_pos = 0;
 	read->pPng = NULL;
 	read->pInfo = NULL;
 	read->row_pointer = NULL;
+	read->fp = NULL;
+	read->buffer = NULL;
+	read->length = 0;
+	read->read_pos = 0;
 
 	g_signal_connect( out, "close", 
 		G_CALLBACK( read_destroy ), read ); 
 
-        if( !(read->fp = vips__file_open_read( name, NULL, FALSE )) ) 
-		return( NULL );
-
 	if( !(read->pPng = png_create_read_struct( 
 		PNG_LIBPNG_VER_STRING, NULL,
 		user_error_function, user_warning_function )) ) 
@@ -166,6 +190,22 @@ read_new( const char *name, VipsImage *out )
 	if( !(read->pInfo = png_create_info_struct( read->pPng )) ) 
 		return( NULL );
 
+	return( read );
+}
+
+static Read *
+read_new_filename( VipsImage *out, const char *name, gboolean readbehind )
+{
+	Read *read;
+
+	if( !(read = read_new( out, readbehind )) )
+		return( NULL );
+
+	read->name = vips_strdup( VIPS_OBJECT( out ), name );
+
+        if( !(read->fp = vips__file_open_read( name, NULL, FALSE )) ) 
+		return( NULL );
+
 	/* Read enough of the file that png_get_interlace_type() will start
 	 * working.
 	 */
@@ -213,32 +253,22 @@ png2vips_header( Read *read, VipsImage *out )
 
 	/* png_get_channels() gives us 1 band for palette images ... so look
 	 * at colour_type for output bands.
+	 *
+	 * Ignore alpha, we detect that separately below.
 	 */
 	switch( color_type ) {
 	case PNG_COLOR_TYPE_PALETTE: 
 		bands = 3; 
-
-		/* If there's transparency in the palette, we make an alpha.
-		 */
-		if( png_get_valid( read->pPng, read->pInfo, PNG_INFO_tRNS ) ) 
-			bands = 4; 
-
 		break;
 
+	case PNG_COLOR_TYPE_GRAY_ALPHA: 
 	case PNG_COLOR_TYPE_GRAY: 
 		bands = 1; 
 		break;
 
-	case PNG_COLOR_TYPE_GRAY_ALPHA: 
-		bands = 2; 
-		break;
-
 	case PNG_COLOR_TYPE_RGB: 
-		bands = 3; 
-		break;
-
 	case PNG_COLOR_TYPE_RGB_ALPHA: 
-		bands = 4; 
+		bands = 3; 
 		break;
 
 	default:
@@ -264,17 +294,18 @@ png2vips_header( Read *read, VipsImage *out )
 	if( color_type == PNG_COLOR_TYPE_PALETTE )
 		png_set_palette_to_rgb( read->pPng );
 
-	/* Expand transparency images too.
+	/* Expand transparency.
 	 */
 	if( png_get_valid( read->pPng, read->pInfo, PNG_INFO_tRNS ) ) {
 		png_set_tRNS_to_alpha( read->pPng );
-
-		/* Some PNGs have an alpha but do not set color_type correctly
-		 * .. make sure we add space for an alpha.
+		bands += 1;
+	}
+	else if( color_type == PNG_COLOR_TYPE_GRAY_ALPHA || 
+		color_type == PNG_COLOR_TYPE_RGB_ALPHA ) {
+		/* Some images have no transparency chunk, but still set
+		 * color_type to alpha.
 		 */
-		if( color_type == PNG_COLOR_TYPE_GRAY ||
-			color_type == PNG_COLOR_TYPE_RGB )
-			bands += 1;
+		bands += 1;
 	}
 
 	/* Expand <8 bit images to full bytes.
@@ -317,10 +348,10 @@ png2vips_header( Read *read, VipsImage *out )
 		VIPS_CODING_NONE, interpretation, 
 		Xres, Yres );
 
-	/* We're always supposed to set dhint.
+	/* Sequential mode needs thinstrip to work with things like
+	 * vips_shrink().
 	 */
-        vips_demand_hint( out, 
-		VIPS_DEMAND_STYLE_FATSTRIP, NULL );
+        vips_demand_hint( out, VIPS_DEMAND_STYLE_THINSTRIP, NULL );
 
 	/* Fetch the ICC profile. @name is useless, something like "icc" or
 	 * "ICC Profile" etc.  Ignore it.
@@ -345,6 +376,16 @@ png2vips_header( Read *read, VipsImage *out )
 			(VipsCallbackFn) vips_free, profile_copy, proflen );
 	}
 
+	/* Sanity-check line size.
+	 */
+	png_read_update_info( read->pPng, read->pInfo );
+	if( png_get_rowbytes( read->pPng, read->pInfo ) != 
+		VIPS_IMAGE_SIZEOF_LINE( out ) ) {
+		vips_error( "vipspng", 
+			"%s", _( "unable to read PNG header" ) );
+		return( -1 );
+	}
+
 	return( 0 );
 }
 
@@ -355,7 +396,7 @@ vips__png_header( const char *name, VipsImage *out )
 {
 	Read *read;
 
-	if( !(read = read_new( name, out )) ||
+	if( !(read = read_new_filename( out, name, FALSE )) ||
 		png2vips_header( read, out ) ) 
 		return( -1 );
 
@@ -410,18 +451,34 @@ png2vips_generate( VipsRegion *or,
 	g_assert( r->width == or->im->Xsize );
 	g_assert( VIPS_RECT_BOTTOM( r ) <= or->im->Ysize );
 
-	if( setjmp( png_jmpbuf( read->pPng ) ) ) {
-#ifdef DEBUG
-		printf( "png2vips_generate: failing in setjmp\n" ); 
-#endif /*DEBUG*/
+	/* Tiles should always be a strip in height, unless it's the final
+	 * strip.
+	 */
+	g_assert( r->height == VIPS_MIN( 8, or->im->Ysize - r->top ) ); 
 
-		return( -1 );
-	}
+	/* And check that y_pos is correct. It should be, since we are inside
+	 * a vips_sequential().
+	 */
+	g_assert( r->top == read->y_pos ); 
 
 	for( y = 0; y < r->height; y++ ) {
 		png_bytep q = (png_bytep) VIPS_REGION_ADDR( or, 0, r->top + y );
 
-		png_read_row( read->pPng, q, NULL );
+		/* We need to catch and ignore errors from read_row().
+		 */
+		if( !setjmp( png_jmpbuf( read->pPng ) ) ) 
+			png_read_row( read->pPng, q, NULL );
+		else { 
+#ifdef DEBUG
+			printf( "png2vips_generate: png_read_row() failed, "
+				"line %d\n", r->top + y ); 
+			printf( "png2vips_generate: file %s\n", read->name );
+			printf( "png2vips_generate: thread %p\n", 
+				g_thread_self() );
+#endif /*DEBUG*/
+		}
+
+		read->y_pos += 1;
 	}
 
 	return( 0 );
@@ -438,7 +495,7 @@ vips__png_isinterlaced( const char *filename )
 	int interlace_type;
 
 	image = vips_image_new();
-	if( !(read = read_new( filename, image )) ) {
+	if( !(read = read_new_filename( image, filename, FALSE )) ) {
 		g_object_unref( image );
 		return( -1 );
 	}
@@ -448,24 +505,13 @@ vips__png_isinterlaced( const char *filename )
 	return( interlace_type != PNG_INTERLACE_NONE );
 }
 
-int
-vips__png_read( const char *name, VipsImage *out )
+static int
+png2vips_image( Read *read, VipsImage *out )
 {
+	int interlace_type = png_get_interlace_type( read->pPng, read->pInfo );
 	VipsImage **t = (VipsImage **) 
 		vips_object_local_array( VIPS_OBJECT( out ), 3 );
 
-	Read *read;
-	int interlace_type;
-
-#ifdef DEBUG
-	printf( "png2vips: reading \"%s\"\n", name );
-#endif /*DEBUG*/
-
-	if( !(read = read_new( name, out )) )
-		return( -1 );
-
-	interlace_type = png_get_interlace_type( read->pPng, read->pInfo );
-
 	if( interlace_type != PNG_INTERLACE_NONE ) { 
 		/* Arg awful interlaced image. We have to load to a huge mem 
 		 * buffer, then copy to out.
@@ -482,14 +528,34 @@ vips__png_read( const char *name, VipsImage *out )
 			vips_image_generate( t[0], 
 				NULL, png2vips_generate, NULL, 
 				read, NULL ) ||
-			vips_sequential( t[0], &t[1], NULL ) ||
-			vips_foreign_tilecache( t[1], &t[2], 16 ) || 
-			vips_image_write( t[2], out ) )
+			vips_sequential( t[0], &t[1], 
+				"tile_height", 8,
+				"access", read->readbehind ? 
+					VIPS_ACCESS_SEQUENTIAL : 
+					VIPS_ACCESS_SEQUENTIAL_UNBUFFERED,
+				NULL ) ||
+			vips_image_write( t[1], out ) )
 			return( -1 );
 	}
 
+	return( 0 );
+}
+
+int
+vips__png_read( const char *filename, VipsImage *out, gboolean readbehind )
+{
+	Read *read;
+
 #ifdef DEBUG
-	printf( "png2vips: done\n" );
+	printf( "vips__png_read: reading \"%s\"\n", filename );
+#endif /*DEBUG*/
+
+	if( !(read = read_new_filename( out, filename, readbehind )) ||
+		png2vips_image( read, out ) )
+		return( -1 ); 
+
+#ifdef DEBUG
+	printf( "vips__png_read: done\n" );
 #endif /*DEBUG*/
 
 	return( 0 );
@@ -504,6 +570,69 @@ vips__png_ispng( const char *filename )
 		!png_sig_cmp( buf, 0, 8 ) );
 }
 
+static void
+vips_png_read_buffer( png_structp pPng, png_bytep data, png_size_t length )
+{
+	Read *read = png_get_io_ptr( pPng ); 
+
+#ifdef DEBUG
+	printf( "vips_png_read_buffer: read %zd bytes\n", length ); 
+#endif /*DEBUG*/
+
+	if( read->read_pos + length > read->length )
+		png_error( pPng, "not enough data in buffer" );
+
+	memcpy( data, read->buffer + read->read_pos, length );
+	read->read_pos += length;
+}
+
+static Read *
+read_new_buffer( VipsImage *out, char *buffer, size_t length, 
+	gboolean readbehind )
+{
+	Read *read;
+
+	if( !(read = read_new( out, readbehind )) )
+		return( NULL );
+
+	read->length = length;
+	read->buffer = buffer;
+
+	png_set_read_fn( read->pPng, read, vips_png_read_buffer ); 
+
+	/* Read enough of the file that png_get_interlace_type() will start
+	 * working.
+	 */
+	png_read_info( read->pPng, read->pInfo );
+
+	return( read );
+}
+
+int
+vips__png_header_buffer( char *buffer, size_t length, VipsImage *out )
+{
+	Read *read;
+
+	if( !(read = read_new_buffer( out, buffer, length, FALSE )) ||
+		png2vips_header( read, out ) ) 
+		return( -1 );
+
+	return( 0 );
+}
+
+int
+vips__png_read_buffer( char *buffer, size_t length, VipsImage *out, 
+	gboolean readbehind  )
+{
+	Read *read;
+
+	if( !(read = read_new_buffer( out, buffer, length, readbehind )) ||
+		png2vips_image( read, out ) )
+		return( -1 ); 
+
+	return( 0 );
+}
+
 const char *vips__png_suffs[] = { ".png", NULL };
 
 /* What we track during a PNG write.
@@ -599,8 +728,6 @@ write_vips( Write *write, int compress, int interlace )
 	int color_type;
 	int interlace_type;
 	int i, nb_passes;
-	void *profile;
-	size_t profile_length;
 
         g_assert( in->BandFmt == VIPS_FORMAT_UCHAR || 
 		in->BandFmt == VIPS_FORMAT_USHORT );
@@ -636,6 +763,10 @@ write_vips( Write *write, int compress, int interlace )
 
 	default:
 		g_assert( 0 );
+
+		/* Keep -Wall happy.
+		 */
+		return( 0 );
 	}
 
 	interlace_type = interlace ? PNG_INTERLACE_ADAM7 : PNG_INTERLACE_NONE;
@@ -652,8 +783,14 @@ write_vips( Write *write, int compress, int interlace )
 
 	/* Set ICC Profile.
 	 */
-	if( !vips_image_get_blob( in, VIPS_META_ICC_NAME, 
-		&profile, &profile_length ) ) {
+	if( vips_image_get_typeof( in, VIPS_META_ICC_NAME ) ) {
+		void *profile;
+		size_t profile_length;
+
+		if( vips_image_get_blob( in, VIPS_META_ICC_NAME, 
+			&profile, &profile_length ) ) 
+			return( -1 ); 
+
 #ifdef DEBUG
 		printf( "write_vips: attaching %zd bytes of ICC profile\n",
 			profile_length );
@@ -698,6 +835,10 @@ vips__png_write( VipsImage *in, const char *filename,
 {
 	Write *write;
 
+#ifdef DEBUG
+	printf( "vips__png_write: writing \"%s\"\n", filename );
+#endif /*DEBUG*/
+
 	if( !(write = write_new( in )) )
 		return( -1 );
 
@@ -718,6 +859,10 @@ vips__png_write( VipsImage *in, const char *filename,
 
 	write_finish( write );
 
+#ifdef DEBUG
+	printf( "vips__png_write: done\n" ); 
+#endif /*DEBUG*/
+
 	return( 0 );
 }
 
