@@ -22,7 +22,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -46,44 +47,6 @@ extern "C" {
 #define VIPS_MAGIC_INTEL (0xb6a6f208U)
 #define VIPS_MAGIC_SPARC (0x08f2a6b6U)
 
-/** 
- * VipsDemandStyle:
- * @VIPS_DEMAND_STYLE_SMALLTILE: demand in small (typically 64x64 pixel) tiles
- * @VIPS_DEMAND_STYLE_FATSTRIP: demand in fat (typically 10 pixel high) strips
- * @VIPS_DEMAND_STYLE_THINSTRIP: demand in thin (typically 1 pixel high) strips
- * @VIPS_DEMAND_STYLE_ANY: demand geometry does not matter
- *
- * See vips_demand_hint(). Operations can hint to the VIPS image IO system about
- * the kind of demand geometry they prefer. 
- *
- * These demand styles are given below in order of increasing
- * restrictiveness.  When demanding output from a pipeline, 
- * vips_image_generate()
- * will use the most restrictive of the styles requested by the operations 
- * in the pipeline.
- *
- * #VIPS_DEMAND_STYLE_THINSTRIP --- This operation would like to output strips 
- * the width of the image and a few pels high. This is option suitable for 
- * point-to-point operations, such as those in the arithmetic package.
- *
- * This option is only efficient for cases where each output pel depends 
- * upon the pel in the corresponding position in the input image.
- *
- * #VIPS_DEMAND_STYLE_FATSTRIP --- This operation would like to output strips 
- * the width of the image and as high as possible. This option is suitable 
- * for area operations which do not violently transform coordinates, such 
- * as im_conv(). 
- *
- * #VIPS_DEMAND_STYLE_SMALLTILE --- This is the most general demand format.
- * Output is demanded in small (around 100x100 pel) sections. This style works 
- * reasonably efficiently, even for bizzare operations like 45 degree rotate.
- *
- * #VIPS_DEMAND_STYLE_ANY --- This image is not being demand-read from a disc 
- * file (even indirectly) so any demand style is OK. It's used for things like
- * im_black() where the pixels are calculated.
- *
- * See also: vips_demand_hint().
- */
 typedef enum {
 	VIPS_DEMAND_STYLE_ERROR = -1,	
 	VIPS_DEMAND_STYLE_SMALLTILE,	
@@ -107,77 +70,28 @@ typedef enum {
 	VIPS_IMAGE_PARTIAL		/* partial image */
 } VipsImageType;
 
-/**
- * VipsInterpretation: 
- * @VIPS_INTERPREATION_MULTIBAND: generic many-band image
- * @VIPS_INTERPREATION_B_W: some kind of single-band image
- * @VIPS_INTERPREATION_HISTOGRAM: a 1D image such as a histogram or lookup table
- * @VIPS_INTERPREATION_FOURIER: image is in fourier space
- * @VIPS_INTERPREATION_XYZ: the first three bands are CIE XYZ 
- * @VIPS_INTERPREATION_LAB: pixels are in CIE Lab space
- * @VIPS_INTERPREATION_CMYK: the first four bands are in CMYK space
- * @VIPS_INTERPREATION_LABQ: implies #VIPS_CODING_LABQ
- * @VIPS_INTERPREATION_RGB: generic RGB space
- * @VIPS_INTERPREATION_UCS: a uniform colourspace based on CMC
- * @VIPS_INTERPREATION_LCH: pixels are in CIE LCh space
- * @VIPS_INTERPREATION_LABS: CIE LAB coded as three signed 16-bit values
- * @VIPS_INTERPREATION_sRGB: pixels are sRGB
- * @VIPS_INTERPREATION_YXY: pixels are CIE Yxy
- * @VIPS_INTERPREATION_RGB16: generic 16-bit RGB
- * @VIPS_INTERPREATION_GREY16: generic 16-bit mono
- * @VIPS_INTERPREATION_ARRAY: an array
- *
- * How the values in an image should be interpreted. For example, a
- * three-band float image of type #VIPS_INTERPREATION_LAB should have its pixels
- * interpreted as coordinates in CIE Lab space.
- *
- * These values are set by operations as hints to user-interfaces built on top 
- * of VIPS to help them show images to the user in a meaningful way. 
- * Operations do not use these values to decide their action.
- *
- * The gaps in the numbering are historical and must be maintained. Allocate 
- * new numbers from the end.
- */
 typedef enum {
 	VIPS_INTERPRETATION_ERROR = -1,
 	VIPS_INTERPRETATION_MULTIBAND = 0,
 	VIPS_INTERPRETATION_B_W = 1,
 	VIPS_INTERPRETATION_HISTOGRAM = 10,
-	VIPS_INTERPRETATION_FOURIER = 24,
 	VIPS_INTERPRETATION_XYZ = 12,
 	VIPS_INTERPRETATION_LAB = 13,
 	VIPS_INTERPRETATION_CMYK = 15,
 	VIPS_INTERPRETATION_LABQ = 16,
 	VIPS_INTERPRETATION_RGB = 17,
-	VIPS_INTERPRETATION_UCS = 18,
+	VIPS_INTERPRETATION_CMC = 18,
 	VIPS_INTERPRETATION_LCH = 19,
 	VIPS_INTERPRETATION_LABS = 21,
 	VIPS_INTERPRETATION_sRGB = 22,
 	VIPS_INTERPRETATION_YXY = 23,
+	VIPS_INTERPRETATION_FOURIER = 24,
 	VIPS_INTERPRETATION_RGB16 = 25,
 	VIPS_INTERPRETATION_GREY16 = 26,
-	VIPS_INTERPRETATION_ARRAY = 27
+	VIPS_INTERPRETATION_MATRIX = 27,
+	VIPS_INTERPRETATION_scRGB = 28
 } VipsInterpretation;
 
-/**
- * VipsBandFormat: 
- * @VIPS_FORMAT_NOTSET: invalid setting
- * @VIPS_FORMAT_UCHAR: unsigned char format
- * @VIPS_FORMAT_CHAR: char format
- * @VIPS_FORMAT_USHORT: unsigned short format
- * @VIPS_FORMAT_SHORT: short format
- * @VIPS_FORMAT_UINT: unsigned int format
- * @VIPS_FORMAT_INT: int format
- * @VIPS_FORMAT_FLOAT: float format
- * @VIPS_FORMAT_COMPLEX: complex (two floats) format
- * @VIPS_FORMAT_DOUBLE: double float format
- * @VIPS_FORMAT_DPCOMPLEX: double complex (two double) format
- *
- * The format used for each band element. 
- *
- * Each corresponnds to a native C type for the current machine. For example,
- * #VIPS_FORMAT_USHORT is <type>unsigned short</type>.
- */
 typedef enum {
 	VIPS_FORMAT_NOTSET = -1,
 	VIPS_FORMAT_UCHAR = 0,
@@ -193,21 +107,6 @@ typedef enum {
 	VIPS_FORMAT_LAST = 10
 } VipsBandFormat;
 
-/**
- * VipsCoding: 
- * @VIPS_CODING_NONE: pixels are not coded
- * @VIPS_CODING_LABQ: pixels encode 3 float CIELAB values as 4 uchar
- * @VIPS_CODING_RAD: pixels encode 3 float RGB as 4 uchar (Radiance coding)
- *
- * How pixels are coded. 
- *
- * Normally, pixels are uncoded and can be manipulated as you would expect.
- * However some file formats code pixels for compression, and sometimes it's
- * useful to be able to manipulate images in the coded format.
- *
- * The gaps in the numbering are historical and must be maintained. Allocate 
- * new numbers from the end.
- */
 typedef enum {
 	VIPS_CODING_ERROR = -1,
 	VIPS_CODING_NONE = 0,
@@ -216,6 +115,13 @@ typedef enum {
 	VIPS_CODING_LAST = 7
 } VipsCoding;
 
+typedef enum {
+	VIPS_ACCESS_RANDOM,
+	VIPS_ACCESS_SEQUENTIAL,
+	VIPS_ACCESS_SEQUENTIAL_UNBUFFERED,
+	VIPS_ACCESS_LAST
+} VipsAccess;
+
 /* Struct we keep a record of execution time in. Passed to eval signal so
  * it can assess progress.
  */
@@ -419,6 +325,16 @@ typedef struct _VipsImageClass {
 	 */
 	void (*invalidate)( VipsImage *image );
 
+	/* Minimise this pipeline. 
+	 *
+	 * This is triggered (sometimes) at the end of eval to signal that
+	 * we're probably done and that operations involved should try to
+	 * minimise memory use by, for example, dropping caches. 
+	 *
+	 * See vips_tilecache().
+	 */
+	void (*minimise)( VipsImage *image );
+
 } VipsImageClass;
 
 GType vips_image_get_type( void );
@@ -447,9 +363,10 @@ extern const guint64 vips__image_sizeof_bandformat[];
  */
 #ifdef VIPS_DEBUG
 #define VIPS_IMAGE_ADDR( I, X, Y ) \
-	( ((X) >= 0 && (X) < (I)->Xsize && \
-	   (Y) >= 0 && (Y) < (I)->Ysize) ? \
-	     ((I)->data + \
+	( ((X) >= 0 && (X) < VIPS_IMAGE( I )->Xsize && \
+	   (Y) >= 0 && (Y) < VIPS_IMAGE( I )->Ysize && \
+	   VIPS_IMAGE( I )->data) ? \
+	     (VIPS_IMAGE( I )->data + \
 	       (Y) * VIPS_IMAGE_SIZEOF_LINE( I ) + \
 	       (X) * VIPS_IMAGE_SIZEOF_PEL( I )) : \
 	     (fprintf( stderr, \
@@ -461,8 +378,8 @@ extern const guint64 vips__image_sizeof_bandformat[];
 		__FILE__, __LINE__, \
 		(X), (Y), \
 		0, 0, \
-		(I)->Xsize, \
-		(I)->Ysize ), abort(), (VipsPel *) NULL) \
+		VIPS_IMAGE( I )->Xsize, \
+		VIPS_IMAGE( I )->Ysize ), (VipsPel *) NULL) \
 	)
 #else /*!VIPS_DEBUG*/
 #define VIPS_IMAGE_ADDR( I, X, Y ) \
@@ -471,16 +388,30 @@ extern const guint64 vips__image_sizeof_bandformat[];
 	 (X) * VIPS_IMAGE_SIZEOF_PEL( I ))
 #endif /*VIPS_DEBUG*/
 
+#ifdef VIPS_DEBUG
+#define VIPS_MATRIX( I, X, Y ) \
+	((VIPS_IMAGE( I )->BandFmt == VIPS_FORMAT_DOUBLE && \
+	  VIPS_IMAGE( I )->Bands == 1) ? \
+	 ((double *) VIPS_IMAGE_ADDR( I, X, Y )) : \
+	 (fprintf( stderr, "VIPS_MATRIX: not a matrix image\n" ), \
+	  	(double *) NULL)) 
+#else /*!VIPS_DEBUG*/
+#define VIPS_MATRIX( I, X, Y ) \
+	((double *) VIPS_IMAGE_ADDR( I, X, Y ))
+#endif /*VIPS_DEBUG*/
+
 int vips_image_written( VipsImage *image );
 
 void vips_image_invalidate_all( VipsImage *image );
 
+void vips_image_minimise_all( VipsImage *image );
+
 void vips_image_preeval( VipsImage *image );
 void vips_image_eval( VipsImage *image, guint64 processed );
 void vips_image_posteval( VipsImage *image );
 void vips_image_set_progress( VipsImage *image, gboolean progress );
 
-gboolean vips_image_get_kill( VipsImage *image );
+gboolean vips_image_iskilled( VipsImage *image );
 void vips_image_set_kill( VipsImage *image, gboolean kill );
 
 VipsImage *vips_image_new( void );
@@ -491,10 +422,11 @@ VipsImage *vips_image_new_from_file_raw( const char *filename,
 	int xsize, int ysize, int bands, guint64 offset );
 VipsImage *vips_image_new_from_memory( void *buffer, 
 	int xsize, int ysize, int bands, VipsBandFormat bandfmt );
-VipsImage *vips_image_new_array( int xsize, int ysize );
+VipsImage *vips_image_new_matrix( int width, int height );
+VipsImage *vips_image_new_matrixv( int width, int height, ... );
 void vips_image_set_delete_on_close( VipsImage *image, 
 	gboolean delete_on_close );
-VipsImage *vips_image_new_disc_temp( const char *format );
+VipsImage *vips_image_new_temp_file( const char *format );
 int vips_image_write( VipsImage *image, VipsImage *out );
 int vips_image_write_to_file( VipsImage *image, const char *filename );
 
@@ -511,9 +443,13 @@ int vips_image_write_line( VipsImage *image, int ypos, VipsPel *linebuffer );
 
 gboolean vips_band_format_isint( VipsBandFormat format );
 gboolean vips_band_format_isuint( VipsBandFormat format );
+gboolean vips_band_format_is8bit( VipsBandFormat format );
 gboolean vips_band_format_isfloat( VipsBandFormat format );
 gboolean vips_band_format_iscomplex( VipsBandFormat format );
 
+int vips_system( const char *cmd_format, ... )
+	__attribute__((sentinel));
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
