@@ -5,19 +5,20 @@
 
     Copyright (C) 1991-2005 The National Gallery
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
+    This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -36,6 +37,13 @@ extern "C" {
 
 #include <vips/vips.h>
 
+typedef enum /*< flags >*/ {
+	VIPS_OPERATION_NONE = 0,
+	VIPS_OPERATION_SEQUENTIAL = 1,
+	VIPS_OPERATION_SEQUENTIAL_UNBUFFERED = 2,
+	VIPS_OPERATION_NOCACHE = 4
+} VipsOperationFlags;
+
 #define VIPS_TYPE_OPERATION (vips_operation_get_type())
 #define VIPS_OPERATION( obj ) \
 	(G_TYPE_CHECK_INSTANCE_CAST( (obj), \
@@ -74,15 +82,22 @@ typedef struct _VipsOperationClass {
 	/* Print the usage message.
 	 */
 	void (*usage)( struct _VipsOperationClass *, VipsBuf * );
+
+	/* Return a set of operation flags. 
+	 */
+	VipsOperationFlags (*get_flags)( VipsOperation * ); 
+	VipsOperationFlags flags;
 } VipsOperationClass;
 
 GType vips_operation_get_type( void );
 
+VipsOperationFlags vips_operation_get_flags( VipsOperation *operation );
 void vips_operation_class_print_usage( VipsOperationClass *operation_class );
 
 int vips_operation_call_valist( VipsOperation *operation, va_list ap );
 VipsOperation *vips_operation_new( const char *name ); 
-int vips_call( const char *operation_name, ... );
+int vips_call( const char *operation_name, ... )
+	__attribute__((sentinel));
 int vips_call_split( const char *operation_name, va_list optional, ... );
 
 void vips_call_options( GOptionGroup *group, VipsOperation *operation );
@@ -91,6 +106,7 @@ int vips_call_argv( VipsOperation *operation, int argc, char **argv );
 void vips_cache_drop_all( void );
 int vips_cache_operation_buildp( VipsOperation **operation );
 VipsOperation *vips_cache_operation_build( VipsOperation *operation );
+void vips_cache_print( void );
 void vips_cache_set_max( int max );
 void vips_cache_set_max_mem( size_t max_mem );
 int vips_cache_get_max( void );
