@@ -64,7 +64,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -128,13 +129,12 @@ extern "C" {
 #include <vips/convolution.h>
 #include <vips/morphology.h>
 #include <vips/mosaicing.h>
-#include <vips/histograms_lut.h>
+#include <vips/histogram.h>
 #include <vips/freq_filt.h>
 #include <vips/resample.h>
 #include <vips/colour.h>
-#include <vips/disp.h>
 #include <vips/inplace.h>
-#include <vips/other.h>
+#include <vips/create.h>
 #include <vips/video.h>
 #include <vips/cimg_funcs.h>
 
