@@ -20,7 +20,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -31,7 +32,7 @@
  */
 
 #ifndef VIPS_VIPS7COMPAT_H
-#define VIPS_VIP7COMPATS_H
+#define VIPS_VIPS7COMPAT_H
 
 #ifdef __cplusplus
 extern "C" {
@@ -40,6 +41,62 @@ extern "C" {
 /* Renamed types.
  */
 
+#define IM_D93_X0 VIPS_D93_X0 
+#define IM_D93_Y0 VIPS_D93_Y0 
+#define IM_D93_Z0 VIPS_D93_Z0 
+
+#define IM_D75_X0 VIPS_D75_X0 
+#define IM_D75_Y0 VIPS_D75_Y0 
+#define IM_D75_Z0 VIPS_D75_Z0 
+
+#define IM_D65_X0 VIPS_D65_X0 
+#define IM_D65_Y0 VIPS_D65_Y0 
+#define IM_D65_Z0 VIPS_D65_Z0 
+
+#define IM_D55_X0 VIPS_D55_X0 
+#define IM_D55_Y0 VIPS_D55_Y0 
+#define IM_D55_Z0 VIPS_D55_Z0 
+
+#define IM_D50_X0 VIPS_D50_X0 
+#define IM_D50_Y0 VIPS_D50_Y0 
+#define IM_D50_Z0 VIPS_D50_Z0 
+
+#define IM_A_X0 VIPS_A_X0 
+#define IM_A_Y0 VIPS_A_Y0 
+#define IM_A_Z0 VIPS_A_Z0 
+
+#define IM_B_X0 VIPS_B_X0 
+#define IM_B_Y0 VIPS_B_Y0 
+#define IM_B_Z0 VIPS_B_Z0 
+
+#define IM_C_X0 VIPS_C_X0 
+#define IM_C_Y0 VIPS_C_Y0 
+#define IM_C_Z0 VIPS_C_Z0 
+
+#define IM_E_X0 VIPS_E_X0 
+#define IM_E_Y0 VIPS_E_Y0 
+#define IM_E_Z0 VIPS_E_Z0 
+
+#define IM_D3250_X0 VIPS_D3250_X0 
+#define IM_D3250_Y0 VIPS_D3250_Y0 
+#define IM_D3250_Z0 VIPS_D3250_Z0 
+
+#define im_col_Lab2XYZ vips_col_Lab2XYZ
+#define im_col_XYZ2Lab vips_col_XYZ2Lab
+#define im_col_ab2h vips_col_ab2h
+#define im_col_ab2Ch vips_col_ab2Ch
+#define im_col_Ch2ab vips_col_Ch2ab
+
+#define im_col_L2Lucs vips_col_L2Lcmc
+#define im_col_C2Cucs vips_col_C2Ccmc
+#define im_col_Ch2hucs vips_col_Ch2hcmc
+#define im_col_pythagoras vips_pythagoras
+
+#define im_col_make_tables_UCS vips_col_make_tables_CMC
+#define im_col_Lucs2L vips_col_Lcmc2L
+#define im_col_Cucs2C vips_col_Ccmc2C
+#define im_col_Chucs2h vips_col_Chcmc2h
+
 #define PEL VipsPel
 
 #define IM_BANDFMT_NOTSET VIPS_FORMAT_NOTSET
@@ -74,7 +131,7 @@ extern "C" {
 #define IM_TYPE_CMYK VIPS_INTERPRETATION_CMYK
 #define IM_TYPE_LABQ VIPS_INTERPRETATION_LABQ
 #define IM_TYPE_RGB VIPS_INTERPRETATION_RGB
-#define IM_TYPE_UCS VIPS_INTERPRETATION_UCS
+#define IM_TYPE_UCS VIPS_INTERPRETATION_CMC
 #define IM_TYPE_LCH VIPS_INTERPRETATION_LCH
 #define IM_TYPE_LABS VIPS_INTERPRETATION_LABS
 #define IM_TYPE_sRGB VIPS_INTERPRETATION_sRGB
@@ -86,6 +143,11 @@ extern "C" {
 #define IMAGE VipsImage
 #define REGION VipsRegion
 
+#define IM_INTENT_PERCEPTUAL VIPS_INTENT_PERCEPTUAL
+#define IM_INTENT_RELATIVE_COLORIMETRIC VIPS_INTENT_RELATIVE
+#define IM_INTENT_SATURATION VIPS_INTENT_SATURATION
+#define IM_INTENT_ABSOLUTE_COLORIMETRIC VIPS_INTENT_ABSOLUTE
+
 /* Renamed macros.
  */
 
@@ -134,14 +196,13 @@ extern "C" {
 #define im_error_clear vips_error_clear
 #define im_warn vips_warn
 #define im_vwarn vips_vwarn
-#define im_diag vips_diag
-#define im_vdiag vips_vdiag
+#define im_diag vips_info
+#define im_vdiag vips_vinfo
 #define error_exit vips_error_exit
 
 #define im_get_argv0 vips_get_argv0
 #define im_version_string vips_version_string
 #define im_version vips_version
-#define im_init_world vips_init
 #define im_get_option_group vips_get_option_group
 #define im_guess_prefix vips_guess_prefix
 #define im_guess_libdir vips_guess_libdir
@@ -152,7 +213,7 @@ extern "C" {
 #define im_cp_desc_array vips_image_copy_fields_array
 #define im_image vips_image_new_from_memory
 #define im_binfile vips_image_new_from_file_raw
-#define im__open_temp vips_image_new_disc_temp
+#define im__open_temp vips_image_new_temp_file
 #define im__test_kill( I ) (vips_image_get_kill( I ))
 #define im__start_eval( I ) (vips_image_preeval( I ), vips_image_get_kill( I ))
 #define im__handle_eval( I, W, H ) \
@@ -194,6 +255,8 @@ extern "C" {
 /* Compat functions.
  */
 
+int im_init_world( const char *argv0 ); 
+
 VipsImage *im_open( const char *filename, const char *mode );
 
 VipsImage *im_open_local( VipsImage *parent, 
@@ -373,6 +436,10 @@ G_STMT_START { \
 
 #define im__change_suffix vips__change_suffix
 
+int vips_check_coding_labq( const char *domain, VipsImage *im );
+int vips_check_coding_rad( const char *domain, VipsImage *im );
+int vips_check_bands_3ormore( const char *domain, VipsImage *im );
+
 /* Buffer processing.
  */
 typedef void (*im_wrapone_fn)( void *in, void *out, int width,
@@ -404,7 +471,8 @@ int im_wrapmany( VipsImage **in, VipsImage *out,
 
 #define im_header_int vips_image_get_int
 #define im_header_double vips_image_get_double
-#define im_header_string vips_image_get_string
+#define im_header_string( IMAGE, FIELD, STRING ) \
+	vips_image_get_string( IMAGE, FIELD, (const char **) STRING )
 #define im_header_as_string vips_image_get_as_string
 #define im_header_get_typeof vips_image_get_typeof
 #define im_header_get vips_image_get
@@ -599,6 +667,20 @@ int im_lesseqconst( VipsImage *in, VipsImage *out, double c );
 int im_moreconst( VipsImage *in, VipsImage *out, double c );
 int im_moreeqconst( VipsImage *in, VipsImage *out, double c );
 
+int im_maxpos_vec( VipsImage *im, int *xpos, int *ypos, double *maxima, int n );
+int im_minpos_vec( VipsImage *im, int *xpos, int *ypos, double *minima, int n );
+
+int im_maxpos_avg( VipsImage *im, double *xpos, double *ypos, double *out );
+
+int im_linreg( VipsImage **ins, VipsImage *out, double *xs );
+
+int im_cross_phase( VipsImage *a, VipsImage *b, VipsImage *out );
+
+int im_point( VipsImage *im, VipsInterpolate *interpolate, 
+	double x, double y, int band, double *out );
+int im_point_bilinear( VipsImage *im, 
+	double x, double y, int band, double *out );
+
 int im_copy( VipsImage *in, VipsImage *out );
 int im_copy_set( VipsImage *in, VipsImage *out, 
 	VipsInterpretation interpretation, 
@@ -608,6 +690,7 @@ int im_copy_set_meta( VipsImage *in, VipsImage *out,
 int im_copy_morph( VipsImage *in, VipsImage *out, 
 	int bands, VipsBandFormat format, VipsCoding coding );
 int im_copy_swap( VipsImage *in, VipsImage *out );
+int im_copy_file( VipsImage *in, VipsImage *out );
 int im_copy_native( VipsImage *in, VipsImage *out, gboolean is_msb_first );
 int im_embed( VipsImage *in, VipsImage *out, 
 	int type, int x, int y, int width, int height );
@@ -624,10 +707,51 @@ int im_extract_bands( VipsImage *in, VipsImage *out, int band, int nbands );
 int im_extract_areabands( VipsImage *in, VipsImage *out,
 	int left, int top, int width, int height, int band, int nbands );
 int im_replicate( VipsImage *in, VipsImage *out, int across, int down );
+int im_wrap( VipsImage *in, VipsImage *out, int x, int y );
+int im_rotquad( VipsImage *in, VipsImage *out );
 int im_clip2fmt( VipsImage *in, VipsImage *out, VipsBandFormat fmt );
 int im_bandjoin( VipsImage *in1, VipsImage *in2, VipsImage *out );
 int im_gbandjoin( VipsImage **in, VipsImage *out, int n );
+int im_grid( VipsImage *in, VipsImage *out, int tile_height, int across, int down );
+int im_scale( VipsImage *in, VipsImage *out );
+int im_scaleps( VipsImage *in, VipsImage *out );
+int im_msb( VipsImage *in, VipsImage *out );
+int im_msb_band( VipsImage *in, VipsImage *out, int band );
+int im_zoom( VipsImage *in, VipsImage *out, int xfac, int yfac );
+int im_subsample( VipsImage *in, VipsImage *out, int xshrink, int yshrink );
+
+int im_gaussnoise( VipsImage *out, int x, int y, double mean, double sigma );
+int im_text( VipsImage *out, const char *text, const char *font,
+	int width, int alignment, int dpi );
 int im_black( VipsImage *out, int x, int y, int bands );
+int im_make_xy( VipsImage *out, const int xsize, const int ysize );
+int im_zone( VipsImage *out, int size );
+int im_fzone( VipsImage *out, int size );
+int im_feye( VipsImage *out,
+	const int xsize, const int ysize, const double factor );
+int im_eye( VipsImage *out,
+	const int xsize, const int ysize, const double factor );
+int im_grey( VipsImage *out, const int xsize, const int ysize );
+int im_fgrey( VipsImage *out, const int xsize, const int ysize );
+int im_sines( VipsImage *out,
+	int xsize, int ysize, double horfreq, double verfreq );
+int im_buildlut( DOUBLEMASK *input, VipsImage *output );
+int im_invertlut( DOUBLEMASK *input, VipsImage *output, int lut_size );
+int im_identity( VipsImage *lut, int bands );
+int im_identity_ushort( VipsImage *lut, int bands, int sz );
+
+int im_tone_build_range( VipsImage *out,
+	int in_max, int out_max,
+	double Lb, double Lw, double Ps, double Pm, double Ph,
+	double S, double M, double H );
+int im_tone_build( VipsImage *out,
+	double Lb, double Lw, double Ps, double Pm, double Ph,
+	double S, double M, double H );
+
+int im_system( VipsImage *im, const char *cmd, char **out );
+VipsImage *im_system_image( VipsImage *im, 
+	const char *in_format, const char *out_format, const char *cmd_format, 
+	char **log );
 
 int im_c2amph( VipsImage *in, VipsImage *out );
 int im_c2rect( VipsImage *in, VipsImage *out );
@@ -643,6 +767,7 @@ int im_ifthenelse( VipsImage *c, VipsImage *a, VipsImage *b, VipsImage *out );
 int im_blend( VipsImage *c, VipsImage *a, VipsImage *b, VipsImage *out );
 
 DOUBLEMASK *im_vips2mask( VipsImage *in, const char *filename );
+INTMASK *im_vips2imask( IMAGE *in, const char *filename );
 int im_mask2vips( DOUBLEMASK *in, VipsImage *out );
 
 int im_bandmean( VipsImage *in, VipsImage *out );
@@ -650,6 +775,122 @@ int im_recomb( VipsImage *in, VipsImage *out, DOUBLEMASK *recomb );
 
 int im_argb2rgba( VipsImage *in, VipsImage *out );
 
+int im_falsecolour( VipsImage *in, VipsImage *out );
+int im_gammacorrect( VipsImage *in, VipsImage *out, double exponent );
+
+int im_shrink( VipsImage *in, VipsImage *out, double xshrink, double yshrink );
+int im_affinei( VipsImage *in, VipsImage *out, 
+	VipsInterpolate *interpolate,
+	double a, double b, double c, double d, double dx, double dy, 
+	int ox, int oy, int ow, int oh );
+int im_affinei_all( VipsImage *in, VipsImage *out, VipsInterpolate *interpolate,
+	double a, double b, double c, double d, double dx, double dy ) ;
+int im_rightshift_size( VipsImage *in, VipsImage *out, 
+	int xshift, int yshift, int band_fmt );
+
+int im_Lab2XYZ_temp( IMAGE *in, IMAGE *out, double X0, double Y0, double Z0 );
+int im_Lab2XYZ( IMAGE *in, IMAGE *out );
+int im_XYZ2Lab( VipsImage *in, VipsImage *out );
+int im_XYZ2Lab_temp( VipsImage *in, VipsImage *out, 
+	double X0, double Y0, double Z0 );
+int im_Lab2LCh( VipsImage *in, VipsImage *out );
+int im_LCh2Lab( VipsImage *in, VipsImage *out );
+int im_LCh2UCS( VipsImage *in, VipsImage *out );
+int im_UCS2LCh( VipsImage *in, VipsImage *out );
+int im_XYZ2Yxy( VipsImage *in, VipsImage *out );
+int im_Yxy2XYZ( VipsImage *in, VipsImage *out );
+int im_float2rad( VipsImage *in, VipsImage *out );
+int im_rad2float( VipsImage *in, VipsImage *out );
+int im_Lab2LabQ( VipsImage *in, VipsImage *out );
+int im_LabQ2Lab( VipsImage *in, VipsImage *out );
+int im_Lab2LabS( VipsImage *in, VipsImage *out );
+int im_LabS2Lab( VipsImage *in, VipsImage *out );
+int im_LabQ2LabS( VipsImage *in, VipsImage *out );
+int im_LabS2LabQ( VipsImage *in, VipsImage *out );
+int im_LabQ2sRGB( VipsImage *in, VipsImage *out );
+
+int im_XYZ2sRGB( IMAGE *in, IMAGE *out );
+int im_sRGB2XYZ( IMAGE *in, IMAGE *out );
+
+struct im_col_display;
+#define im_col_displays(S) (NULL)
+#define im_LabQ2disp_build_table(A, B) (NULL)
+#define im_LabQ2disp_table(A, B, C) (im_LabQ2disp(A, B, C))
+
+int im_Lab2disp( IMAGE *in, IMAGE *out, struct im_col_display *disp );
+int im_disp2Lab( IMAGE *in, IMAGE *out, struct im_col_display *disp );
+
+int im_dE_fromdisp( IMAGE *, IMAGE *, IMAGE *, struct im_col_display * );
+int im_dECMC_fromdisp( IMAGE *, IMAGE *, IMAGE *, struct im_col_display * );
+
+#define im_disp2XYZ(A, B, C) (im_sRGB2XYZ(A, B))
+#define im_XYZ2disp(A, B, C) (im_XYZ2sRGB(A, B))
+#define im_LabQ2disp(A, B, C) (im_LabQ2sRGB(A, B))
+
+int im_icc_transform( VipsImage *in, VipsImage *out, 
+	const char *input_profile_filename,
+	const char *output_profile_filename,
+	VipsIntent intent );
+
+#define im_icc_present vips_icc_present
+
+int im_icc_import( VipsImage *in, VipsImage *out, 
+	const char *input_profile_filename, VipsIntent intent );
+int im_icc_import_embedded( VipsImage *in, VipsImage *out, VipsIntent intent );
+int im_icc_export_depth( VipsImage *in, VipsImage *out, int depth,
+	const char *output_profile_filename, VipsIntent intent );
+int im_icc_ac2rc( VipsImage *in, VipsImage *out, const char *profile_filename );
+
+int im_LabQ2XYZ( VipsImage *in, VipsImage *out );
+int im_UCS2XYZ( VipsImage *in, VipsImage *out );
+int im_UCS2Lab( VipsImage *in, VipsImage *out );
+int im_Lab2UCS( VipsImage *in, VipsImage *out );
+int im_XYZ2UCS( VipsImage *in, VipsImage *out );
+
+int im_dE_fromLab( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_dECMC_fromLab( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_dE_fromXYZ( VipsImage *in1, VipsImage *in2, VipsImage *out );
+int im_dE00_fromLab( VipsImage *in1, VipsImage *in2, VipsImage *out );
+
+int im_lab_morph( VipsImage *in, VipsImage *out,
+	DOUBLEMASK *mask,
+	double L_offset, double L_scale,
+	double a_scale, double b_scale );
+
+#define im_col_dE00 vips_col_dE00
+
+int im_quadratic( IMAGE *in, IMAGE *out, IMAGE *coeff );
+
+int im_maplut( VipsImage *in, VipsImage *out, VipsImage *lut );
+int im_hist( VipsImage *in, VipsImage *out, int bandno );
+int im_histgr( VipsImage *in, VipsImage *out, int bandno );
+int im_histcum( VipsImage *in, VipsImage *out );
+int im_histnorm( VipsImage *in, VipsImage *out );
+int im_histeq( VipsImage *in, VipsImage *out );
+int im_heq( VipsImage *in, VipsImage *out, int bandno );
+int im_histnD( VipsImage *in, VipsImage *out, int bins );
+int im_hist_indexed( VipsImage *index, VipsImage *value, VipsImage *out );
+int im_histplot( VipsImage *in, VipsImage *out );
+int im_project( VipsImage *in, VipsImage *hout, VipsImage *vout );
+int im_profile( IMAGE *in, IMAGE *out, int dir );
+int im_hsp( VipsImage *in, VipsImage *ref, VipsImage *out );
+int im_histspec( VipsImage *in, VipsImage *ref, VipsImage *out );
+int im_lhisteq( VipsImage *in, VipsImage *out, int xwin, int ywin );
+int im_stdif( VipsImage *in, VipsImage *out,
+	double a, double m0, double b, double s0, int xwin, int ywin );
+int im_mpercent( VipsImage *in, double percent, int *out );
+int im_mpercent_hist( VipsImage *hist, double percent, int *out );
+int im_ismonotonic( VipsImage *lut, int *out );
+
+int im_tone_analyse( VipsImage *in, VipsImage *out,
+	double Ps, double Pm, double Ph, double S, double M, double H );
+int im_tone_map( VipsImage *in, VipsImage *out, VipsImage *lut );
+
+/* Not really correct, but who uses these.
+ */
+#define im_lhisteq_raw im_lhisteq
+#define im_stdif_raw im_stdif
+
 /* ruby-vips uses this
  */
 #define vips_class_map_concrete_all vips_class_map_all
