@@ -46,7 +46,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -242,7 +243,7 @@ flood_scanline( Flood *flood, int x, int y, int *x1, int *x2 )
 
 	/* Paint the range we discovered.
 	 */
-	im__draw_scanline( DRAW( flood ), y, *x1, *x2 );
+	im__draw_scanline( draw, y, *x1, *x2 );
 
 	if( flood->dout ) {
 		flood->left = IM_MIN( flood->left, *x1 );
