@@ -1,4 +1,9 @@
 /* cache vips operations
+ *
+ * 20/6/12
+ * 	- try to make it compile on centos5
+ * 7/7/12
+ * 	- add a lock so we can run operations from many threads
  */
 
 /*
@@ -17,7 +22,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -31,9 +37,6 @@
 
    TODO
 
-	should the cache be thread-private? or lock? or say operations can 
-	only be made from the main thread?
-
 	listen for invalidate
 
 	will we need to drop all on exit? unclear
@@ -100,6 +103,24 @@ static GHashTable *vips_cache_table = NULL;
  */
 static int vips_cache_time = 0;
 
+/* Protect cache access with this.
+ */
+static GMutex *vips_cache_lock = NULL;
+
+/* Old versions of glib are missing these. When we abandon centos 5, switch to
+ * g_int64_hash() and g_double_hash().
+ */
+#define INT64_HASH(X) (g_direct_hash(X))
+#define DOUBLE_HASH(X) (g_direct_hash(X))
+
+/* Old glibs use g_value_get_char(), new ones g_value_get_schar().
+ */
+#ifdef HAVE_VALUE_GET_SCHAR
+#define VIPS_VALUE_GET_CHAR g_value_get_schar
+#else 
+#define VIPS_VALUE_GET_CHAR g_value_get_char
+#endif 
+
 /* Pass in the pspec so we can get the generic type. For example, a 
  * held in a GParamSpec allowing OBJECT, but the value could be of type
  * VipsImage. generics are much faster to compare.
@@ -116,7 +137,7 @@ vips_value_hash( GParamSpec *pspec, GValue *value )
 	if( generic == G_TYPE_PARAM_BOOLEAN )
 		return( (unsigned int) g_value_get_boolean( value ) );
 	else if( generic == G_TYPE_PARAM_CHAR )
-		return( (unsigned int) g_value_get_char( value ) );
+		return( (unsigned int) VIPS_VALUE_GET_CHAR( value ) );
 	else if( generic == G_TYPE_PARAM_UCHAR )
 		return( (unsigned int) g_value_get_uchar( value ) );
 	else if( generic == G_TYPE_PARAM_INT )
@@ -134,22 +155,22 @@ vips_value_hash( GParamSpec *pspec, GValue *value )
 	else if( generic == G_TYPE_PARAM_UINT64 ) {
 		guint64 i = g_value_get_uint64( value );
 
-		return( g_int64_hash( (gint64 *) &i ) );
+		return( INT64_HASH( (gint64 *) &i ) );
 	}
 	else if( generic == G_TYPE_PARAM_INT64 ) {
 		gint64 i = g_value_get_int64( value );
 
-		return( g_int64_hash( &i ) );
+		return( INT64_HASH( &i ) );
 	}
 	else if( generic == G_TYPE_PARAM_FLOAT ) {
 		float f = g_value_get_float( value );
 
-		return( *((unsigned int *) &f) );
+		return( g_direct_hash( (void *) &f ) );
 	}
 	else if( generic == G_TYPE_PARAM_DOUBLE ) {
 		double d = g_value_get_double( value );
 
-		return( g_double_hash( &d ) );
+		return( DOUBLE_HASH( &d ) );
 	}
 	else if( generic == G_TYPE_PARAM_STRING ) {
 		const char *s = g_value_get_string( value );
@@ -218,8 +239,8 @@ vips_value_equal( GParamSpec *pspec, GValue *v1, GValue *v2 )
 		return( g_value_get_boolean( v1 ) == 
 			g_value_get_boolean( v2 ) );
 	else if( generic == G_TYPE_PARAM_CHAR ) 
-		return( g_value_get_char( v1 ) ==
-			g_value_get_char( v2 ) );
+		return( VIPS_VALUE_GET_CHAR( v1 ) ==
+			VIPS_VALUE_GET_CHAR( v2 ) );
 	if( generic == G_TYPE_PARAM_UCHAR ) 
 		return( g_value_get_uchar( v1 ) ==
 			g_value_get_uchar( v2 ) );
@@ -253,9 +274,15 @@ vips_value_equal( GParamSpec *pspec, GValue *v1, GValue *v2 )
 	if( generic == G_TYPE_PARAM_DOUBLE ) 
 		return( g_value_get_double( v1 ) ==
 			g_value_get_double( v2 ) );
-	if( generic == G_TYPE_PARAM_STRING ) 
-		return( strcmp( g_value_get_string( v1 ),
-			g_value_get_string( v2 ) ) == 0 );
+	if( generic == G_TYPE_PARAM_STRING ) {
+		const char *s1 = g_value_get_string( v1 );
+		const char *s2 = g_value_get_string( v2 );
+
+		if( s1 == s2 )
+			return( TRUE );
+		else
+			return( s1 && s2 && strcmp( s1, s2 ) == 0 );
+	}
 	if( generic == G_TYPE_PARAM_BOXED ) 
 		return( g_value_get_boxed( v1 ) ==
 			g_value_get_boxed( v2 ) );
@@ -394,10 +421,12 @@ vips_operation_equal( VipsOperation *a, VipsOperation *b )
 	return( FALSE );
 }
 
-static void
-vips_cache_init( void )
+void
+vips__cache_init( void )
 {
 	if( !vips_cache_table ) {
+		vips_cache_lock = vips_g_mutex_new();
+
 		vips_cache_table = g_hash_table_new( 
 			(GHashFunc) vips_operation_hash, 
 			(GEqualFunc) vips_operation_equal );
@@ -416,24 +445,36 @@ vips_cache_init( void )
 	}
 }
 
-static void
-vips_cache_dump( void )
+static void *
+vips_cache_print_fn( void *value, void *a, void *b )
 {
+	char str[32768];
+	VipsBuf buf = VIPS_BUF_STATIC( str );
+
+	vips_object_to_string( VIPS_OBJECT( value ), &buf );
+
+	printf( "%p - %s\n", value, vips_buf_all( &buf ) );
+
+	return( NULL );
+}
+
+/**
+ * vips_cache_print:
+ *
+ * Print the whole operation cache to stdout. Handy for debugging.
+ */
+void
+vips_cache_print( void )
+{
+	g_mutex_lock( vips_cache_lock );
+
 	if( vips_cache_table ) {
-		GHashTableIter iter;
-		gpointer key, value;
-
 		printf( "Operation cache:\n" );
-		g_hash_table_iter_init( &iter, vips_cache_table );
-		while( g_hash_table_iter_next( &iter, &key, &value ) ) {
-			char str[32768];
-			VipsBuf buf = VIPS_BUF_STATIC( str );
-
-			vips_object_to_string( VIPS_OBJECT( key ), &buf );
-
-			printf( "%p - %s\n", key, vips_buf_all( &buf ) );
-		}
+		vips_hash_table_map( vips_cache_table, 
+			vips_cache_print_fn, NULL, NULL ); 
 	}
+
+	g_mutex_unlock( vips_cache_lock );
 }
 
 static void *
@@ -485,6 +526,24 @@ vips_cache_drop( VipsOperation *operation )
 	vips_cache_unref( operation );
 }
 
+static void *
+vips_cache_get_first_fn( void *value, void *a, void *b )
+{
+	return( value );
+}
+
+/* Return the first item.
+ */
+static VipsOperation *
+vips_cache_get_first( void )
+{
+	if( vips_cache_table )
+		return( VIPS_OPERATION( vips_hash_table_map( vips_cache_table, 
+			vips_cache_get_first_fn, NULL, NULL ) ) );
+	else
+		return( NULL ); 
+}
+
 /**
  * vips_cache_drop_all:
  *
@@ -493,49 +552,48 @@ vips_cache_drop( VipsOperation *operation )
 void
 vips_cache_drop_all( void )
 {
+	g_mutex_lock( vips_cache_lock );
+
 	if( vips_cache_table ) {
+		VipsOperation *operation;
+
 		if( vips__cache_dump )
-			vips_cache_dump();
+			vips_cache_print();
 
 		/* We can't modify the hash in the callback from
 		 * g_hash_table_foreach() and friends. Repeatedly drop the
 		 * first item instead.
 		 */
-		for(;;) {
-			GHashTableIter iter;
-			gpointer key, value;
-
-			g_hash_table_iter_init( &iter, vips_cache_table );
-			if( !g_hash_table_iter_next( &iter, &key, &value ) )
-				break;
-
-			vips_cache_drop( (VipsOperation *) key );
-		}
+		while( (operation = vips_cache_get_first()) ) 
+			vips_cache_drop( operation );
 
 		VIPS_FREEF( g_hash_table_unref, vips_cache_table );
 	}
+
+	g_mutex_unlock( vips_cache_lock );
 }
 
 static void
-vips_cache_select_cb( VipsOperation *key, VipsOperation *value, 
+vips_cache_get_lru_cb( VipsOperation *key, VipsOperation *value, 
 	VipsOperation **best )
-
 {
 	if( !*best ||
 		(*best)->time > value->time )
 		*best = value;
 }
 
-/* Find an op to drop ... LRU for now.
+/* Get the least-recently-used cache item. 
+ *
+ * TODO ... will this be too expensive? probably not
  */
 static VipsOperation *
-vips_cache_select( void )
+vips_cache_get_lru( void )
 {
 	VipsOperation *operation;
 
 	operation = NULL;
 	g_hash_table_foreach( vips_cache_table,
-		(GHFunc) vips_cache_select_cb, &operation );
+		(GHFunc) vips_cache_get_lru_cb, &operation );
 
 	return( operation );
 }
@@ -547,17 +605,21 @@ vips_cache_trim( void )
 {
 	VipsOperation *operation;
 
+	g_mutex_lock( vips_cache_lock );
+
 	while( vips_cache_table &&
 		(g_hash_table_size( vips_cache_table ) > vips_cache_max ||
 		vips_tracked_get_files() > vips_cache_max_files ||
 		vips_tracked_get_mem() > vips_cache_max_mem) &&
-		(operation = vips_cache_select()) ) {
+		(operation = vips_cache_get_lru()) ) {
 #ifdef DEBUG
 		printf( "vips_cache_trim: trimming %p\n", operation );
 #endif /*DEBUG*/
 
 		vips_cache_drop( operation );
 	}
+
+	g_mutex_unlock( vips_cache_lock );
 }
 
 static void *
@@ -621,17 +683,18 @@ vips_cache_operation_buildp( VipsOperation **operation )
 
 #ifdef VIPS_DEBUG
 	printf( "vips_cache_operation_build: " );
-	vips_object_print_summary_stdout( VIPS_OBJECT( *operation ) );
+	vips_object_print_dump( VIPS_OBJECT( *operation ) );
 #endif /*VIPS_DEBUG*/
 
-	vips_cache_init();
-
 	vips_cache_trim();
 
+	g_mutex_lock( vips_cache_lock );
+
 	if( (hit = g_hash_table_lookup( vips_cache_table, *operation )) ) {
 		if( vips__cache_trace ) {
-			printf( "vips cache: hit %p\n  ", hit );
-			vips_object_print_summary( VIPS_OBJECT( *operation ) );
+			printf( "vips cache: found old operation\n" );
+			printf( "\t" );
+			vips_object_print_summary( VIPS_OBJECT( hit ) );
 		}
 
 		/* Ref before unref in case *operation == hit.
@@ -641,19 +704,38 @@ vips_cache_operation_buildp( VipsOperation **operation )
 
 		*operation = hit;
 	}
-	else {
+
+	g_mutex_unlock( vips_cache_lock );
+
+	if( !hit ) {
+		if( vips_object_build( VIPS_OBJECT( *operation ) ) ) 
+			return( -1 );
+
+		/* Has to be after _build() so we can see output args.
+		 */
 		if( vips__cache_trace ) {
-			printf( "vips cache: miss %p\n  ", *operation );
+			if( vips_operation_get_flags( *operation ) & 
+				VIPS_OPERATION_NOCACHE )
+				printf( "vips cache: uncacheable\n" );
+			else
+				printf( "vips cache: caching new operation\n" );
+			printf( "\t" );
 			vips_object_print_summary( VIPS_OBJECT( *operation ) );
 		}
 
-		if( vips_object_build( VIPS_OBJECT( *operation ) ) )
-			return( -1 );
+		g_mutex_lock( vips_cache_lock );
 
-		vips_cache_ref( *operation );
-		g_hash_table_insert( vips_cache_table, *operation, *operation );
+		if( !(vips_operation_get_flags( *operation ) & 
+			VIPS_OPERATION_NOCACHE) ) {
+			vips_cache_ref( *operation );
+			g_hash_table_insert( vips_cache_table, 
+				*operation, *operation );
+		}
+
+		g_mutex_unlock( vips_cache_lock );
 	}
 
+
 	return( 0 );
 }
 
@@ -739,10 +821,17 @@ vips_cache_get_max( void )
 int
 vips_cache_get_size( void )
 {
+	guint size;
+
+	g_mutex_lock( vips_cache_lock );
+
+	size = 0;
 	if( vips_cache_table )
-		return( g_hash_table_size( vips_cache_table ) );
-	else
-		return( 0 );
+		size = g_hash_table_size( vips_cache_table );
+
+	g_mutex_unlock( vips_cache_lock );
+
+	return( size );
 }
 
 /**
