@@ -32,7 +32,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -104,14 +105,45 @@
  * supposed to indicate the component which failed.
  */
 
+/* Show info messages. Handy for debugging. 
+ */
+int vips__info = 0;
+
 /* Make global array to keep the error message buffer.
  */
 #define VIPS_MAX_ERROR (10240)
 static char vips_error_text[VIPS_MAX_ERROR] = "";
 static VipsBuf vips_error_buf = VIPS_BUF_STATIC( vips_error_text );
+static int vips_error_freeze_count = 0;
 
-#define IM_DIAGNOSTICS "IM_DIAGNOSTICS"
-#define IM_WARNING "IM_WARNING"
+/**
+ * vips_error_freeze:
+ *
+ * Stop errors being logged. Use vips_error_thaw() to unfreeze. You can
+ * nest freeze/thaw pairs.
+ */
+void
+vips_error_freeze( void )
+{
+	g_mutex_lock( vips__global_lock );
+	g_assert( vips_error_freeze_count >= 0 );
+	vips_error_freeze_count += 1;
+	g_mutex_unlock( vips__global_lock );
+}
+
+/**
+ * vips_error_thaw:
+ *
+ * Reenable error logging. 
+ */
+void
+vips_error_thaw( void )
+{
+	g_mutex_lock( vips__global_lock );
+	vips_error_freeze_count -= 1;
+	g_assert( vips_error_freeze_count >= 0 );
+	g_mutex_unlock( vips__global_lock );
+}
 
 /**
  * vips_error_buffer: 
@@ -135,6 +167,15 @@ vips_error_buffer( void )
 	return( msg );
 }
 
+/* Some systems do not have va_copy() ... this might work (it does on MSVC),
+ * apparently.
+ *
+ * FIXME ... this should be in configure.in
+ */
+#ifndef va_copy
+#define va_copy(d,s) ((d) = (s))
+#endif
+
 /**
  * vips_verror: 
  * @domain: the source of the error
@@ -148,13 +189,31 @@ vips_error_buffer( void )
 void 
 vips_verror( const char *domain, const char *fmt, va_list ap )
 {
+#ifdef VIPS_DEBUG
+{
+	char txt[256];
+	VipsBuf buf = VIPS_BUF_STATIC( txt );
+	va_list ap2;
+
+	vips_buf_appendf( &buf, "%s: ", domain );
+	va_copy( ap2, ap );
+	vips_buf_vappendf( &buf, fmt, ap2 );
+	vips_buf_appends( &buf, "\n" );
+	VIPS_DEBUG_MSG( "vips_verror: %s", vips_buf_all( &buf ) );
+}
+#endif /*VIPS_DEBUG*/
+
 	g_mutex_lock( vips__global_lock );
-	vips_buf_appendf( &vips_error_buf, "%s: ", domain );
-	vips_buf_vappendf( &vips_error_buf, fmt, ap );
-	vips_buf_appends( &vips_error_buf, "\n" );
+	g_assert( vips_error_freeze_count >= 0 );
+	if( !vips_error_freeze_count ) {
+		vips_buf_appendf( &vips_error_buf, "%s: ", domain );
+		vips_buf_vappendf( &vips_error_buf, fmt, ap );
+		vips_buf_appends( &vips_error_buf, "\n" );
+	}
 	g_mutex_unlock( vips__global_lock );
 
-	VIPS_DEBUG_MSG( "vips_verror: %s\n", fmt );
+	if( vips__fatal )
+		vips_error_exit( "vips__fatal" );
 }
 
 /**
@@ -262,6 +321,12 @@ vips_error_g( GError **error )
 	if( !vips_domain ) 
 		vips_domain = g_quark_from_string( "libvips" );
 
+	/* glib does not expect a trailing '\n' and vips always has one.
+	 */
+	g_mutex_lock( vips__global_lock );
+	vips_buf_removec( &vips_error_buf, '\n' );
+	g_mutex_unlock( vips__global_lock );
+
 	g_set_error( error, vips_domain, -1, "%s", vips_error_buffer() );
 	vips_error_clear();
 }
@@ -269,7 +334,7 @@ vips_error_g( GError **error )
 /**
  * vips_error_clear: 
  *
- * Clear and reset the error buffer. This is typically called after presentng
+ * Clear and reset the error buffer. This is typically called after presenting
  * an error to the user.
  *
  * See also: vips_error_buffer().
@@ -283,26 +348,28 @@ vips_error_clear( void )
 }
 
 /**
- * vips_vdiag: 
- * @domain: the source of the diagnostic message
+ * vips_vinfo: 
+ * @domain: the source of the message
  * @fmt: printf()-style format string for the message
  * @ap: arguments to the format string
  *
- * Sends a formatted diagnostic message to stderr. If you define the
- * environment variable IM_DIAGNOSTICS, these message are surpressed.
+ * Sends a formatted informational message to stderr if the --vips-info flag
+ * has been given to the program or the environment variable IM_INFO has been
+ * defined. 
  *
- * Diagnostic messages are used to report details about the operation of
+ * Informational messages are used to report details about the operation of
  * functions.
  *
- * See also: vips_diag(), vips_warn().
+ * See also: vips_info(), vips_warn().
  */
 void 
-vips_vdiag( const char *domain, const char *fmt, va_list ap )
+vips_vinfo( const char *domain, const char *fmt, va_list ap )
 {
-	if( !g_getenv( IM_DIAGNOSTICS ) ) {
+	if( vips__info ) { 
 		g_mutex_lock( vips__global_lock );
-		(void) fprintf( stderr, _( "%s: " ), _( "vips diagnostic" ) );
-		(void) fprintf( stderr, _( "%s: " ), domain );
+		(void) fprintf( stderr, _( "%s: " ), _( "info" ) );
+		if( domain )
+			(void) fprintf( stderr, _( "%s: " ), domain );
 		(void) vfprintf( stderr, fmt, ap );
 		(void) fprintf( stderr, "\n" );
 		g_mutex_unlock( vips__global_lock );
@@ -310,26 +377,27 @@ vips_vdiag( const char *domain, const char *fmt, va_list ap )
 }
 
 /**
- * vips_diag: 
+ * vips_info: 
  * @domain: the source of the diagnostic message
  * @fmt: printf()-style format string for the message
  * @Varargs: arguments to the format string
  *
- * Sends a formatted diagnostic message to stderr. If you define the
- * environment variable IM_DIAGNOSTICS, these message are surpressed.
+ * Sends a formatted informational message to stderr if the --vips-info flag
+ * has been given to the program or the environment variable IM_INFO has been
+ * defined. 
  *
- * Diagnostic messages are used to report details about the operation of
+ * Informational messages are used to report details about the operation of
  * functions.
  *
  * See also: vips_vdiag(), vips_warn().
  */
 void 
-vips_diag( const char *domain, const char *fmt, ... )
+vips_info( const char *domain, const char *fmt, ... )
 {
 	va_list ap;
 
 	va_start( ap, fmt );
-	vips_vdiag( domain, fmt, ap );
+	vips_vinfo( domain, fmt, ap );
 	va_end( ap );
 }
 
@@ -344,19 +412,23 @@ vips_diag( const char *domain, const char *fmt, ... )
  *
  * Warning messages are used to report things like overflow counts.
  *
- * See also: vips_diag(), vips_warn().
+ * See also: vips_info(), vips_warn().
  */
 void 
 vips_vwarn( const char *domain, const char *fmt, va_list ap )
 {	
-	if( !g_getenv( IM_WARNING ) ) {
+	if( !g_getenv( "IM_WARNING" ) ) {
 		g_mutex_lock( vips__global_lock );
 		(void) fprintf( stderr, _( "%s: " ), _( "vips warning" ) );
-		(void) fprintf( stderr, _( "%s: " ), domain );
+		if( domain )
+			(void) fprintf( stderr, _( "%s: " ), domain );
 		(void) vfprintf( stderr, fmt, ap );
 		(void) fprintf( stderr, "\n" );
 		g_mutex_unlock( vips__global_lock );
 	}
+
+	if( vips__fatal )
+		vips_error_exit( "vips__fatal" );
 }
 
 /**
@@ -370,7 +442,7 @@ vips_vwarn( const char *domain, const char *fmt, va_list ap )
  *
  * Warning messages are used to report things like overflow counts.
  *
- * See also: vips_diag(), vips_vwarn().
+ * See also: vips_info(), vips_vwarn().
  */
 void 
 vips_warn( const char *domain, const char *fmt, ... )
@@ -415,7 +487,10 @@ vips_error_exit( const char *fmt, ... )
 
 	vips_shutdown();
 
-	exit( 1 );
+	if( vips__fatal )
+		abort();
+	else
+		exit( 1 );
 }
 
 /**
@@ -463,7 +538,7 @@ vips_check_coding_noneorlabq( const char *domain, VipsImage *im )
 	if( im->Coding != VIPS_CODING_NONE && 
 		im->Coding != VIPS_CODING_LABQ ) {
 		vips_error( domain, 
-			"%s", _( "image coding must be NONE or LABQ" ) );
+			"%s", _( "image coding must be 'none' or 'labq'" ) );
 		return( -1 );
 	}
 
@@ -499,11 +574,12 @@ vips_check_coding_known( const char *domain, VipsImage *im )
 }
 
 /**
- * vips_check_coding_rad:
+ * vips_check_coding:
  * @domain: the originating domain for the error message
  * @im: image to check
+ * @coding: required coding
  *
- * Check that the image is in Radiance coding. 
+ * Check that the image has the required @coding.
  * If not, set an error message
  * and return non-zero.
  *
@@ -512,38 +588,11 @@ vips_check_coding_known( const char *domain, VipsImage *im )
  * Returns: 0 on OK, or -1 on error.
  */
 int
-vips_check_coding_rad( const char *domain, VipsImage *im )
+vips_check_coding( const char *domain, VipsImage *im, VipsCoding coding )
 {
-	if( im->Coding != VIPS_CODING_RAD ||
-		im->BandFmt != VIPS_FORMAT_UCHAR || 
-		im->Bands != 4 ) { 
-		vips_error( domain, "%s", _( "Radiance coding only" ) );
-		return( -1 );
-	}
-
-	return( 0 );
-}
-
-/**
- * vips_check_coding_labq:
- * @domain: the originating domain for the error message
- * @im: image to check
- *
- * Check that the image is in LABQ coding. 
- * If not, set an error message
- * and return non-zero.
- *
- * See also: vips_error().
- *
- * Returns: 0 on OK, or -1 on error.
- */
-int
-vips_check_coding_labq( const char *domain, VipsImage *im )
-{
-	if( im->Coding != VIPS_CODING_LABQ ||
-		im->BandFmt != VIPS_FORMAT_UCHAR || 
-		im->Bands != 4 ) { 
-		vips_error( domain, "%s", _( "LABQ coding only" ) );
+	if( im->Coding != coding ) {
+		vips_error( domain, _( "coding '%s' only" ), 
+			vips_enum_nick( VIPS_TYPE_CODING, coding ) );
 		return( -1 );
 	}
 
@@ -600,7 +649,7 @@ vips_check_bands( const char *domain, VipsImage *im, int bands )
 }
 
 /**
- * vips_check_1or3:
+ * vips_check_bands_1or3:
  * @domain: the originating domain for the error message
  * @im: image to check
  *
@@ -624,6 +673,32 @@ vips_check_bands_1or3( const char *domain, VipsImage *im )
 	return( 0 );
 }
 
+/**
+ * vips_check_bands_atleast:
+ * @domain: the originating domain for the error message
+ * @im: image to check
+ * @bands: at least this many bands
+ *
+ * Check that the image has at least @bands bands. 
+ * Otherwise set an error message
+ * and return non-zero.
+ *
+ * See also: vips_error().
+ *
+ * Returns: 0 if OK, -1 otherwise.
+ */
+int
+vips_check_bands_atleast( const char *domain, VipsImage *im, int bands )
+{
+	if( im->Bands < bands ) {
+		vips_error( domain, 
+			_( "image must have at least %d bands" ), bands );
+		return( -1 );
+	}
+
+	return( 0 );
+}
+
 /**
  * vips_check_bands_1orn:
  * @domain: the originating domain for the error message
@@ -1043,6 +1118,29 @@ vips_check_coding_same( const char *domain, VipsImage *im1, VipsImage *im2 )
 	return( 0 );
 }
 
+/**
+ * vips_check_vector_length:
+ * @domain: the originating domain for the error message
+ * @n: number of elements in vector
+ * @len: number of elements vector should have
+ *
+ * Check that @n == @len. 
+ *
+ * See also: vips_error().
+ *
+ * Returns: 0 if OK, -1 otherwise.
+ */
+int
+vips_check_vector_length( const char *domain, int n, int len )
+{
+	if( n != len ) {
+		vips_error( domain, _( "vector must have %d elements" ), len );
+		return( -1 );
+	}
+
+	return( 0 );
+}
+
 /**
  * vips_check_vector:
  * @domain: the originating domain for the error message
@@ -1099,6 +1197,49 @@ vips_check_hist( const char *domain, VipsImage *im )
 	return( 0 );
 }
 
+/** 
+ * vips_check_matrix: 
+ * @domain: the originating domain for the error message
+ * @im: image to check 
+ * @out: put image as in-memory doubles here
+ *
+ * Matrix images must have width and height less than 100000 and have 1 band.
+ *
+ * Return 0 if the image will pass as a matrix, or -1 and set an error 
+ * message otherwise.
+ *
+ * @out is set to be @im cast to double and stored in memory. Use
+ * VIPS_MATRIX() to address values in @out. 
+ *
+ * You must unref @out when you are done with it.
+ *
+ * See also: VIPS_MATRIX(), vips_object_local()
+ *
+ * Returns: 0 if OK, -1 otherwise.
+ */
+int
+vips_check_matrix( const char *domain, VipsImage *im, VipsImage **out )
+{
+	*out = NULL;
+
+	if( im->Xsize > 100000 || im->Ysize > 100000 ) {
+		vips_error( domain, "%s", _( "matrix image too large" ) );
+		return( -1 );
+	}
+	if( im->Bands != 1 ) {
+		vips_error( domain, 
+			"%s", _( "matrix image must have one band" ) ); 
+		return( -1 );
+	}
+
+	if( vips_cast( im, out, VIPS_FORMAT_DOUBLE, NULL ) )
+                return( -1 );
+        if( vips_image_wio_input( *out ) )
+                return( -1 );
+
+	return( 0 );
+}
+
 /**
  * vips_check_imask: (skip)
  * @domain: the originating domain for the error message
