@@ -38,7 +38,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -364,12 +365,151 @@ vips_image_get_coding( const VipsImage *image )
 	return( image->Coding );
 }
 
+/* vips_image_get_interpretation:
+ * @image: image to guess for
+ *
+ * Return the #VipsInterpretation set in the image header.
+ * Use vips_image_guess_interpretation() if you want a sanity-checked value.
+ *
+ * Returns: the #VipsInterpretation from the image header.
+ */
 VipsInterpretation
 vips_image_get_interpretation( const VipsImage *image )
 {
 	return( image->Type );
 }
 
+/* Try to pick a sane value for interpretation, assuming Type has been set
+ * incorrectly.
+ */
+static VipsInterpretation
+vips_image_default_interpretation( const VipsImage *image )
+{
+	switch( image->Coding ) {
+	case VIPS_CODING_LABQ:
+		return( VIPS_INTERPRETATION_LABQ );
+	case VIPS_CODING_RAD:
+		return( VIPS_INTERPRETATION_RGB );
+	default:
+		break;
+	}
+
+	if( image->Bands == 1 )
+		return( VIPS_INTERPRETATION_B_W );
+	else
+		return( VIPS_INTERPRETATION_MULTIBAND );
+}
+
+/* vips_image_guess_interpretation:
+ * @image: image to guess for
+ *
+ * Return the #VipsInterpretation for an image, guessing a default value if
+ * the set value looks wrong.
+ *
+ * Returns: a sensible #VipsInterpretation for the image.
+ */
+VipsInterpretation
+vips_image_guess_interpretation( const VipsImage *image )
+{
+	gboolean sane;
+
+	sane = TRUE;
+
+	/* Coding overrides interpretation.
+	 */
+	switch( image->Coding ) {
+	case VIPS_CODING_LABQ:
+		if( image->Type != VIPS_INTERPRETATION_LABQ )
+			sane = FALSE;
+		break;
+
+	case VIPS_CODING_RAD:
+		if( image->Type != VIPS_INTERPRETATION_RGB )
+			sane = FALSE;
+		break;
+
+	default:
+		break;
+	}
+
+	switch( image->Type ) {
+	case VIPS_INTERPRETATION_MULTIBAND: 
+		if( image->Bands == 1 )
+			sane = FALSE;
+		break;
+
+	case VIPS_INTERPRETATION_B_W: 
+		if( image->Bands > 1 )
+			sane = FALSE;
+		break;
+
+	case VIPS_INTERPRETATION_HISTOGRAM: 
+		if( image->Xsize > 1 && image->Ysize > 1 )
+			sane = FALSE;
+		break;
+
+	case VIPS_INTERPRETATION_FOURIER: 
+		if( !vips_band_format_iscomplex( image->BandFmt ) )
+			sane = FALSE;
+		break;
+
+	case VIPS_INTERPRETATION_XYZ: 
+	case VIPS_INTERPRETATION_LAB: 
+	case VIPS_INTERPRETATION_RGB: 
+	case VIPS_INTERPRETATION_CMC: 
+	case VIPS_INTERPRETATION_LCH: 
+	case VIPS_INTERPRETATION_sRGB: 
+	case VIPS_INTERPRETATION_scRGB: 
+	case VIPS_INTERPRETATION_YXY: 
+		if( image->Bands < 3 )
+			sane = FALSE;
+		break;
+
+	case VIPS_INTERPRETATION_CMYK: 
+		if( image->Bands < 4 )
+			sane = FALSE;
+		break;
+
+	case  VIPS_INTERPRETATION_LABQ:
+		if( image->Coding != VIPS_CODING_LABQ )
+			sane = FALSE;
+		break;
+
+	case  VIPS_INTERPRETATION_LABS:
+		if( image->BandFmt != VIPS_FORMAT_SHORT )
+			sane = FALSE;
+		break;
+
+	case  VIPS_INTERPRETATION_RGB16:
+		if( image->BandFmt == VIPS_FORMAT_CHAR ||
+			image->BandFmt == VIPS_FORMAT_UCHAR ||
+			image->Bands < 3 )
+			sane = FALSE;
+		break;
+
+	case  VIPS_INTERPRETATION_GREY16:
+		if( image->BandFmt == VIPS_FORMAT_CHAR ||
+			image->BandFmt == VIPS_FORMAT_UCHAR )
+			sane = FALSE;
+		break;
+
+	case  VIPS_INTERPRETATION_MATRIX:
+		if( image->Bands != 1 )
+			sane = FALSE;
+		break;
+
+	default:
+		g_assert( 0 );
+		sane = FALSE;
+		break;
+	}
+
+	if( sane )
+		return( vips_image_get_interpretation( image ) );
+	else
+		return( vips_image_default_interpretation( image ) );
+}
+
 double
 vips_image_get_xres( const VipsImage *image )
 {
@@ -406,6 +546,30 @@ vips_image_get_mode( const VipsImage *image )
 	return( image->mode );
 }
 
+double
+vips_image_get_scale( const VipsImage *array )
+{
+	double scale;
+
+	scale = 1.0;
+	if( vips_image_get_typeof( array, "scale" ) ) 
+		vips_image_get_double( array, "scale", &scale );
+
+	return( scale );
+}
+
+double
+vips_image_get_offset( const VipsImage *array )
+{
+	double offset;
+
+	offset = 0.0;
+	if( vips_image_get_typeof( array, "offset" ) ) 
+		vips_image_get_double( array, "offset", &offset );
+
+	return( offset );
+}
+
 /**
  * vips_image_get_data: (skip)
  * @image: image to get data for
@@ -524,7 +688,7 @@ meta_cp( VipsImage *dst, const VipsImage *src )
  * on @out.
  *
  * Image history is completely copied from all @in. @out will have the history
- * of all the intput images.
+ * of all the input images.
  *
  * See also: vips_image_copy_fieldsv(), vips_image_copy_fields().
  *
@@ -717,7 +881,7 @@ vips_image_set( VipsImage *image, const char *field, GValue *value )
  * Returns: 0 on success, -1 otherwise.
  */
 int
-vips_image_get( VipsImage *image, const char *field, GValue *value_copy )
+vips_image_get( const VipsImage *image, const char *field, GValue *value_copy )
 {
 	int i;
 	VipsMeta *meta;
@@ -797,7 +961,7 @@ vips_image_get( VipsImage *image, const char *field, GValue *value_copy )
  * field of that name.
  */
 GType 
-vips_image_get_typeof( VipsImage *image, const char *field )
+vips_image_get_typeof( const VipsImage *image, const char *field )
 {
 	int i;
 	VipsMeta *meta;
@@ -939,7 +1103,7 @@ vips_image_set_area( VipsImage *image, const char *field,
 }
 
 static int
-meta_get_value( VipsImage *image,
+meta_get_value( const VipsImage *image,
 	const char *field, GType type, GValue *value_copy )
 {
 	if( vips_image_get( image, field, value_copy ) )
@@ -974,7 +1138,7 @@ meta_get_value( VipsImage *image,
  * Returns: 0 on success, -1 otherwise.
  */
 int
-vips_image_get_area( VipsImage *image, const char *field, void **data )
+vips_image_get_area( const VipsImage *image, const char *field, void **data )
 {
 	GValue value_copy = { 0 };
 
@@ -1031,7 +1195,7 @@ vips_image_set_blob( VipsImage *image, const char *field,
  * Returns: 0 on success, -1 otherwise.
  */
 int
-vips_image_get_blob( VipsImage *image, const char *field, 
+vips_image_get_blob( const VipsImage *image, const char *field, 
 	void **data, size_t *length )
 {
 	GValue value_copy = { 0 };
@@ -1059,7 +1223,7 @@ vips_image_get_blob( VipsImage *image, const char *field,
  * Returns: 0 on success, -1 otherwise.
  */
 int
-vips_image_get_int( VipsImage *image, const char *field, int *out )
+vips_image_get_int( const VipsImage *image, const char *field, int *out )
 {
 	int i;
 	GValue value_copy = { 0 };
@@ -1125,7 +1289,7 @@ vips_image_set_int( VipsImage *image, const char *field, int i )
  * Returns: 0 on success, -1 otherwise.
  */
 int
-vips_image_get_double( VipsImage *image, const char *field, double *out )
+vips_image_get_double( const VipsImage *image, const char *field, double *out )
 {
 	int i;
 	GValue value_copy = { 0 };
@@ -1193,7 +1357,8 @@ vips_image_set_double( VipsImage *image, const char *field, double d )
  * Returns: 0 on success, -1 otherwise.
  */
 int
-vips_image_get_string( VipsImage *image, const char *field, char **out )
+vips_image_get_string( const VipsImage *image, const char *field, 
+	const char **out )
 {
 	int i;
 	GValue value_copy = { 0 };
@@ -1256,7 +1421,8 @@ vips_image_set_string( VipsImage *image, const char *field, const char *str )
  * Returns: 0 on success, -1 otherwise.
  */
 int
-vips_image_get_as_string( VipsImage *image, const char *field, char **out )
+vips_image_get_as_string( const VipsImage *image, 
+	const char *field, char **out )
 {
 	GValue value = { 0 };
 	GType type;
