@@ -2,6 +2,8 @@
  * 
  * 4/2/11
  * 	- hacked up from various places
+ * 6/6/13
+ * 	- vips_image_write() didn't ref non-partial sources
  */
 
 /*
@@ -20,7 +22,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -77,6 +80,132 @@
  * The first four bytes of a VIPS file in SPARC byte ordering.
  */
 
+/** 
+ * VipsAccess:
+ * @VIPS_ACCESS_RANDOM: can read anywhere
+ * @VIPS_ACCESS_SEQUENTIAL: top-to-bottom reading only, but with a small buffer
+ * @VIPS_ACCESS_SEQUENTIAL_UNBUFFERED: top-to-bottom reading only
+ *
+ * The type of access an operation has to supply. See vips_tilecache()
+ * and #VipsForeign. 
+ *
+ * @VIPS_ACCESS_RANDOM means requests can come in any order. 
+ *
+ * @VIPS_ACCESS_SEQUENTIAL means requests will be top-to-bottom, but with some
+ * amount of buffering behind the read point for small non-local accesses. 
+ *
+ * @VIPS_ACCESS_SEQUENTIAL_UNBUFFERED means requests will be strictly
+ * top-to-bottom with no read-behind. This can save some memory. 
+ */
+
+/** 
+ * VipsDemandStyle:
+ * @VIPS_DEMAND_STYLE_SMALLTILE: demand in small (typically 64x64 pixel) tiles
+ * @VIPS_DEMAND_STYLE_FATSTRIP: demand in fat (typically 10 pixel high) strips
+ * @VIPS_DEMAND_STYLE_THINSTRIP: demand in thin (typically 1 pixel high) strips
+ * @VIPS_DEMAND_STYLE_ANY: demand geometry does not matter
+ *
+ * See vips_demand_hint(). Operations can hint to the VIPS image IO system about
+ * the kind of demand geometry they prefer. 
+ *
+ * These demand styles are given below in order of increasing
+ * restrictiveness.  When demanding output from a pipeline, 
+ * vips_image_generate()
+ * will use the most restrictive of the styles requested by the operations 
+ * in the pipeline.
+ *
+ * #VIPS_DEMAND_STYLE_THINSTRIP --- This operation would like to output strips 
+ * the width of the image and a few pels high. This is option suitable for 
+ * point-to-point operations, such as those in the arithmetic package.
+ *
+ * This option is only efficient for cases where each output pel depends 
+ * upon the pel in the corresponding position in the input image.
+ *
+ * #VIPS_DEMAND_STYLE_FATSTRIP --- This operation would like to output strips 
+ * the width of the image and as high as possible. This option is suitable 
+ * for area operations which do not violently transform coordinates, such 
+ * as im_conv(). 
+ *
+ * #VIPS_DEMAND_STYLE_SMALLTILE --- This is the most general demand format.
+ * Output is demanded in small (around 100x100 pel) sections. This style works 
+ * reasonably efficiently, even for bizzare operations like 45 degree rotate.
+ *
+ * #VIPS_DEMAND_STYLE_ANY --- This image is not being demand-read from a disc 
+ * file (even indirectly) so any demand style is OK. It's used for things like
+ * im_black() where the pixels are calculated.
+ *
+ * See also: vips_demand_hint().
+ */
+
+/**
+ * VipsInterpretation: 
+ * @VIPS_INTERPRETATION_MULTIBAND: generic many-band image
+ * @VIPS_INTERPRETATION_B_W: some kind of single-band image
+ * @VIPS_INTERPRETATION_HISTOGRAM: a 1D image, eg. histogram or lookup table
+ * @VIPS_INTERPRETATION_FOURIER: image is in fourier space
+ * @VIPS_INTERPRETATION_XYZ: the first three bands are CIE XYZ 
+ * @VIPS_INTERPRETATION_LAB: pixels are in CIE Lab space
+ * @VIPS_INTERPRETATION_CMYK: the first four bands are in CMYK space
+ * @VIPS_INTERPRETATION_LABQ: implies #VIPS_CODING_LABQ
+ * @VIPS_INTERPRETATION_RGB: generic RGB space
+ * @VIPS_INTERPRETATION_CMC: a uniform colourspace based on CMC(1:1)
+ * @VIPS_INTERPRETATION_LCH: pixels are in CIE LCh space
+ * @VIPS_INTERPRETATION_LABS: CIE LAB coded as three signed 16-bit values
+ * @VIPS_INTERPRETATION_sRGB: pixels are sRGB
+ * @VIPS_INTERPRETATION_scRGB: pixels are scRGB
+ * @VIPS_INTERPRETATION_YXY: pixels are CIE Yxy
+ * @VIPS_INTERPRETATION_RGB16: generic 16-bit RGB
+ * @VIPS_INTERPRETATION_GREY16: generic 16-bit mono
+ * @VIPS_INTERPRETATION_MATRIX: a matrix
+ *
+ * How the values in an image should be interpreted. For example, a
+ * three-band float image of type #VIPS_INTERPRETATION_LAB should have its 
+ * pixels interpreted as coordinates in CIE Lab space.
+ *
+ * These values are set by operations as hints to user-interfaces built on top 
+ * of VIPS to help them show images to the user in a meaningful way. 
+ * Operations do not use these values to decide their action.
+ *
+ * The gaps in the numbering are historical and must be maintained. Allocate 
+ * new numbers from the end.
+ */
+
+/**
+ * VipsBandFormat: 
+ * @VIPS_FORMAT_NOTSET: invalid setting
+ * @VIPS_FORMAT_UCHAR: unsigned char format
+ * @VIPS_FORMAT_CHAR: char format
+ * @VIPS_FORMAT_USHORT: unsigned short format
+ * @VIPS_FORMAT_SHORT: short format
+ * @VIPS_FORMAT_UINT: unsigned int format
+ * @VIPS_FORMAT_INT: int format
+ * @VIPS_FORMAT_FLOAT: float format
+ * @VIPS_FORMAT_COMPLEX: complex (two floats) format
+ * @VIPS_FORMAT_DOUBLE: double float format
+ * @VIPS_FORMAT_DPCOMPLEX: double complex (two double) format
+ *
+ * The format used for each band element. 
+ *
+ * Each corresponnds to a native C type for the current machine. For example,
+ * #VIPS_FORMAT_USHORT is <type>unsigned short</type>.
+ */
+
+/**
+ * VipsCoding: 
+ * @VIPS_CODING_NONE: pixels are not coded
+ * @VIPS_CODING_LABQ: pixels encode 3 float CIELAB values as 4 uchar
+ * @VIPS_CODING_RAD: pixels encode 3 float RGB as 4 uchar (Radiance coding)
+ *
+ * How pixels are coded. 
+ *
+ * Normally, pixels are uncoded and can be manipulated as you would expect.
+ * However some file formats code pixels for compression, and sometimes it's
+ * useful to be able to manipulate images in the coded format.
+ *
+ * The gaps in the numbering are historical and must be maintained. Allocate 
+ * new numbers from the end.
+ */
+
 /** 
  * VipsProgress:
  * @run: Time we have been running 
@@ -139,7 +268,8 @@
  * @X: x coordinate
  * @Y: y coordinate
  *
- * This macro returns a pointer to a pixel in an image. It only works for
+ * This macro returns a pointer to a pixel in an image, cast to a #VipsPel *. 
+ * It only works for
  * images which are fully available in memory, so memory buffers and small
  * mapped images only.
  * 
@@ -150,6 +280,25 @@
  * Returns: The address of pixel (x,y) in the image.
  */
 
+/**
+ * VIPS_MATRIX:
+ * @I: a #VipsImage
+ * @X: x coordinate
+ * @Y: y coordinate
+ *
+ * This macro returns a pointer to a pixel in an image, cast to a double*. The
+ * image must have a single band, be #VIPS_FORMAT_DOUBLE and be 
+ * fully available in memory, so memory buffers and small
+ * mapped images only.
+ * 
+ * If VIPS_DEBUG is defined, you get a version that checks bounds and image
+ * type for you.
+ *
+ * See also: VIPS_IMAGE_ADDR().
+ *
+ * Returns: The address of pixel (x,y) in the image.
+ */
+
 /* Our signals. 
  */
 enum {
@@ -158,6 +307,7 @@ enum {
 	SIG_POSTEVAL,		
 	SIG_WRITTEN,		
 	SIG_INVALIDATE,		
+	SIG_MINIMISE,		
 	SIG_LAST
 };
 
@@ -240,7 +390,7 @@ vips_image_finalize( GObject *gobject )
 	 */
 	vips_image_delete( image );
 
-	VIPS_FREEF( g_mutex_free, image->sslock );
+	VIPS_FREEF( vips_g_mutex_free, image->sslock );
 
 	VIPS_FREE( image->Hist );
 	VIPS_FREEF( vips__gslist_gvalue_free, image->history_list );
@@ -285,9 +435,6 @@ vips_image_dispose( GObject *gobject )
 	if( image->fd != -1 ) {
 		VIPS_DEBUG_MSG( "vips_image_dispose: closing output file\n" );
 
-		if( image->dtype == VIPS_IMAGE_OPENOUT )
-			(void) vips__writehist( image );
-
 		if( vips_tracked_close( image->fd ) == -1 ) 
 			vips_error( "VipsImage", 
 				"%s", _( "unable to close fd" ) );
@@ -310,7 +457,7 @@ vips_image_new_from_file_object( const char *string )
 	image = VIPS_IMAGE( g_object_new( VIPS_TYPE_IMAGE, NULL ) );
 	g_object_set( image,
 		"filename", string,
-		"mode", "rd",
+		"mode", "r",
 		NULL );
 
 	return( VIPS_OBJECT( image ) );
@@ -381,6 +528,9 @@ vips_image_dump( VipsObject *object, VipsBuf *buf )
 		vips_enum_nick( VIPS_TYPE_INTERPRETATION, 
 			vips_image_get_interpretation( image ) ) );
 
+	vips_buf_appendf( buf, ", %s", 
+		vips_enum_nick( VIPS_TYPE_IMAGE_TYPE, image->dtype ) );
+
 	VIPS_OBJECT_CLASS( vips_image_parent_class )->dump( object, buf );
 
 	vips_buf_appendf( buf, "\n" );
@@ -394,6 +544,7 @@ static void
 vips_image_summary( VipsObject *object, VipsBuf *buf )
 {
 	VipsImage *image = VIPS_IMAGE( object );
+	const char *p;
 
 	vips_buf_appendf( buf, "%dx%d",
 		vips_image_get_width( image ), vips_image_get_height( image ) );
@@ -415,6 +566,10 @@ vips_image_summary( VipsObject *object, VipsBuf *buf )
 				vips_image_get_coding( image ) ) );
 	}
 
+	if( vips_image_get_typeof( image, VIPS_META_LOADER ) &&
+		!vips_image_get_string( image, VIPS_META_LOADER, &p ) ) 
+		vips_buf_appendf( buf, ", %s", p );
+
 	VIPS_OBJECT_CLASS( vips_image_parent_class )->summary( object, buf );
 }
 
@@ -457,14 +612,19 @@ vips_image_sanity( VipsObject *object, VipsBuf *buf )
 				image->Coding != VIPS_CODING_NONE && 
 				image->Coding != VIPS_CODING_LABQ &&
 				image->Coding != VIPS_CODING_RAD) ||
-			image->Type > VIPS_INTERPRETATION_ARRAY ||
+			image->Type > VIPS_INTERPRETATION_scRGB ||
 			image->dtype > VIPS_IMAGE_PARTIAL || 
 			image->dhint > VIPS_DEMAND_STYLE_ANY ) 
 			vips_buf_appends( buf, "bad enum\n" );
-		if( image->Xres < 0 || image->Xres < 0 ) 
+		if( image->Xres < 0 || 
+			image->Yres < 0 ) 
 			vips_buf_appends( buf, "bad resolution\n" );
 	}
 
+	/* Must lock around inter-image links.
+	 */
+	g_mutex_lock( vips__global_lock );
+
 	if( vips_slist_map2( image->upstream, 
 		(VipsSListMap2Fn) vips_image_sanity_upstream, image, NULL ) )
 		vips_buf_appends( buf, "upstream broken\n" );
@@ -472,6 +632,8 @@ vips_image_sanity( VipsObject *object, VipsBuf *buf )
 		(VipsSListMap2Fn) vips_image_sanity_downstream, image, NULL ) )
 		vips_buf_appends( buf, "downstream broken\n" );
 
+	g_mutex_unlock( vips__global_lock );
+
 	VIPS_OBJECT_CLASS( vips_image_parent_class )->sanity( object, buf );
 }
 
@@ -508,7 +670,7 @@ vips_image_rewind( VipsObject *object )
 static void
 vips_image_save_cb( VipsImage *image, int *result )
 {
-	if( vips_foreign_save_options( image, image->filename ) )
+	if( vips_foreign_save_options( image, image->filename, NULL ) )
 		*result = -1;
 }
 
@@ -526,8 +688,10 @@ vips_image_preeval_cb( VipsImage *image, VipsProgress *progress, int *last )
 
 	vips_get_tile_size( image, 
 		&tile_width, &tile_height, &nlines );
-	printf( _( "%s %s: %d threads, %d x %d tiles, groups of %d scanlines" ),
+	printf( _( "%s %s: %d x %d pixels, %d threads, %d x %d tiles, "
+		"%d lines in buffer" ),
 		g_get_prgname(), image->filename,
+		image->Xsize, image->Ysize,
 		vips_concurrency_get(),
 		tile_width, tile_height, nlines );
 	printf( "\n" );
@@ -556,8 +720,9 @@ vips_image_posteval_cb( VipsImage *image, VipsProgress *progress )
 {
 	/* Spaces at end help to erase the %complete message we overwrite.
 	 */
-	printf( _( "%s %s: done in %ds          \n" ), 
-		g_get_prgname(), image->filename, progress->run );
+	printf( _( "%s %s: done in %.3gs          \n" ), 
+		g_get_prgname(), image->filename, 
+		g_timer_elapsed( progress->start, NULL ) );
 
 	return( 0 );
 }
@@ -604,7 +769,8 @@ vips_image_build( VipsObject *object )
 	 */
 	switch( mode[0] ) {
         case 'v':
-		/* Used by 'r' for native open of vips, see below.
+		/* Used by 'r' for native open of vips, see below. Also by
+		 * vips_image_rewind_output().
 		 */
 		if( vips_image_open_input( image ) )
 			return( -1 );
@@ -654,8 +820,17 @@ vips_image_build( VipsObject *object )
 		else {
 			VipsImage *t;
 
-			if( vips_foreign_load_options( filename, &t ) )
-				return( -1 );
+			if( mode[1] == 's' ) {
+				if( vips_foreign_load_options( filename, &t, 
+					"sequential", TRUE,
+					NULL ) )
+					return( -1 );
+			}
+			else {
+				if( vips_foreign_load_options( filename, &t, 
+					NULL ) )
+					return( -1 );
+			}
 
 			image->dtype = VIPS_IMAGE_PARTIAL;
 			if( vips_image_write( t, image ) ) {
@@ -791,6 +966,24 @@ vips_image_real_invalidate( VipsImage *image )
 	g_mutex_unlock( image->sslock );
 }
 
+static void 
+vips_image_real_minimise( VipsImage *image )
+{
+	VIPS_DEBUG_MSG( "vips_image_real_minimise: %p\n", image );
+}
+
+static void 
+vips_image_real_written( VipsImage *image, int *result )
+{
+	VIPS_DEBUG_MSG( "vips_image_real_written: %p\n", image );
+
+	/* For vips image write, append the xml after the data.
+	 */
+	if( image->dtype == VIPS_IMAGE_OPENOUT &&
+		vips__writehist( image ) ) 
+		*result = -1;
+}
+
 static void
 vips_image_class_init( VipsImageClass *class )
 {
@@ -826,6 +1019,8 @@ vips_image_class_init( VipsImageClass *class )
 	vobject_class->build = vips_image_build;
 
 	class->invalidate = vips_image_real_invalidate;
+	class->written = vips_image_real_written;
+	class->minimise = vips_image_real_minimise;
 
 	/* Create properties.
 	 */
@@ -846,21 +1041,21 @@ vips_image_class_init( VipsImageClass *class )
 		_( "Image width in pixels" ),
 		VIPS_ARGUMENT_SET_ALWAYS,
 		G_STRUCT_OFFSET( VipsImage, Xsize ),
-		0, 1000000, 0 );
+		1, 1000000000, 1 );
 
 	VIPS_ARG_INT( class, "height", 3, 
 		_( "Height" ), 
 		_( "Image height in pixels" ),
 		VIPS_ARGUMENT_SET_ALWAYS,
 		G_STRUCT_OFFSET( VipsImage, Ysize ),
-		0, 1000000, 0 );
+		1, 1000000000, 1 );
 
 	VIPS_ARG_INT( class, "bands", 4, 
 		_( "Bands" ), 
 		_( "Number of bands in image" ),
 		VIPS_ARGUMENT_SET_ALWAYS,
 		G_STRUCT_OFFSET( VipsImage, Bands ),
-		0, 1000000, 0 );
+		1, 1000000000, 1 );
 
 	VIPS_ARG_ENUM( class, "format", 5, 
 		_( "Format" ), 
@@ -996,6 +1191,15 @@ vips_image_class_init( VipsImageClass *class )
 		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0 );
+
+	vips_image_signals[SIG_MINIMISE] = g_signal_new( "minimise",
+		G_TYPE_FROM_CLASS( class ),
+		G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		G_STRUCT_OFFSET( VipsImageClass, minimise ), 
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0 );
+
 }
 
 static void
@@ -1007,11 +1211,15 @@ vips_image_init( VipsImage *image )
 	 */
 	image->magic = vips_amiMSBfirst() ? VIPS_MAGIC_SPARC : VIPS_MAGIC_INTEL;
 
+	image->Xsize = 1;
+	image->Ysize = 1;
+	image->Bands = 1;
+
 	image->Xres = 1.0;
 	image->Yres = 1.0;
 
 	image->fd = -1;			/* since 0 is stdout */
-	image->sslock = g_mutex_new();
+	image->sslock = vips_g_mutex_new();
 
 	image->sizeof_header = VIPS_SIZEOF_HEADER;
 
@@ -1051,16 +1259,50 @@ vips_image_invalidate_all_cb( VipsImage *image )
  * vips_image_invalidate_all:
  * @image: #VipsImage to invalidate
  *
- * Invalidate all pixel caches on an @image and any derived images. The 
- * "invalidate" callback is triggered for all invalidated images.
+ * Invalidate all pixel caches on an @image and any downstream images, that
+ * is, images which depend on this image. 
+ *
+ * The "invalidate" callback is triggered for all invalidated images.
  */
 void
 vips_image_invalidate_all( VipsImage *image )
 {
-	(void) vips__link_map( image, 
+	(void) vips__link_map( image, FALSE,
 		(VipsSListMap2Fn) vips_image_invalidate_all_cb, NULL, NULL );
 }
 
+void
+vips_image_minimise( VipsImage *image )
+{
+	VIPS_DEBUG_MSG( "vips_image_minimise: %p\n", image );
+
+	g_signal_emit( image, vips_image_signals[SIG_MINIMISE], 0 );
+}
+
+static void *
+vips_image_minimise_all_cb( VipsImage *image )
+{
+	vips_image_minimise( image );
+
+	return( NULL );
+}
+
+/**
+ * vips_image_minimise_all:
+ * @image: #VipsImage to minimise
+ *
+ * Minimise memory use on this image and any upstream images, that is, images
+ * which this image depends upon. 
+ *
+ * The "minimise" callback is triggered for all minimised images.
+ */
+void 
+vips_image_minimise_all( VipsImage *image )
+{
+	(void) vips__link_map( image, TRUE,
+		(VipsSListMap2Fn) vips_image_minimise_all_cb, NULL, NULL );
+}
+
 /* Attach a new time struct, if necessary, and reset it.
  */
 static int
@@ -1200,7 +1442,7 @@ vips_image_set_progress( VipsImage *image, gboolean progress )
 }
 
 gboolean
-vips_image_get_kill( VipsImage *image )
+vips_image_iskilled( VipsImage *image )
 {
 	gboolean kill;
 
@@ -1209,7 +1451,7 @@ vips_image_get_kill( VipsImage *image )
 	/* Has kill been set for this image? If yes, abort evaluation.
 	 */
 	if( image->kill ) {
-		VIPS_DEBUG_MSG( "vips_image_get_kill: %s (%p) killed\n", 
+		VIPS_DEBUG_MSG( "vips_image_iskilled: %s (%p) killed\n", 
 			image->filename, image );
 		vips_error( "VipsImage", 
 			_( "killed for image \"%s\"" ), image->filename );
@@ -1306,29 +1548,21 @@ vips_image_new( void )
  *       VIPS format for your machine, vips_image_new_mode() 
  *       automatically converts the file for you. 
  *
- *       For some large files (eg. TIFF) this may 
- *       not be what you want, it can fill memory very quickly. Instead, you
- *       can either use "rd" mode (see below), or you can use the lower-level 
+ *       Some formats (eg. tiled tiff) are read directly. 
+ *
+ *       Some formats (eg. strip tiff) do not support random access and can't
+ *       be processed directly. Small images are decompressed to memory and
+ *       loaded from there, large images are decompressed to a disc file and
+ *       processed from that.
+ *
+ *       If the operations you intend to perform are sequential, that is, they
+ *       operate in a strict top-to-bottom manner, you can use sequential mode
+ *       instead, see "rs" below,
+ *       or you can use the lower-level 
  *       API and control the loading process yourself. See 
  *       #VipsForeign. 
  *
- *       vips_image_new_mode() can read files in most formats.
- *
- *       Note that <emphasis>"r"</emphasis> mode works in at least two stages. 
- *       It should return quickly and let you check header fields. It will
- *       only actually read in pixels when you first access them. 
- *     </para>
- *   </listitem>
- *   <listitem> 
- *     <para>
- *       <emphasis>"rd"</emphasis>
- *	 opens the named file for reading. If the uncompressed image is larger 
- *	 than a threshold and the file format does not support random access, 
- *	 rather than uncompressing to memory, vips_image_new_mode() will 
- *	 uncompress to a temporary disc file. This file will be automatically 
- *	 deleted when the VipsImage is closed.
- *
- *	 See im_system_image() for an explanation of how VIPS selects a
+ *	 See vips_image_new_temp_file() for an explanation of how VIPS selects a
  *	 location for the temporary file.
  *
  *	 The disc threshold can be set with the "--vips-disc-threshold"
@@ -1344,6 +1578,20 @@ vips_image_new( void )
  *
  *       will copy via disc if "fred.tif" is more than 500 Mbytes
  *       uncompressed. The default threshold is 100 MB.
+ *
+ *       Note that <emphasis>"r"</emphasis> mode works in at least two stages. 
+ *       It should return quickly and let you check header fields. It will
+ *       only actually read in pixels when you first access them. 
+ *     </para>
+ *   </listitem>
+ *   <listitem> 
+ *     <para>
+ *       <emphasis>"rs"</emphasis>
+ *	 opens the named file for reading sequentially. It reads the source
+ *	 image top-to-bottom and serves up pixels to the pipeline as required.
+ *	 Provided the operations that connect to the image all demand pixels
+*	 only top-to-bottom as well, everything is fine and you avoid the
+*	 separate decompress stage.
  *     </para>
  *   </listitem>
  *   <listitem> 
@@ -1415,7 +1663,7 @@ vips_image_new_buffer( void )
  * vips_image_new_from_file:
  * @filename: file to open
  *
- * vips_image_new_from_file() opens @filename for reading in mode "rd". See
+ * vips_image_new_from_file() opens @filename for reading in mode "r". See
  * vips_image_new_mode() for details.
  *
  * See also: vips_image_new_mode().
@@ -1425,7 +1673,7 @@ vips_image_new_buffer( void )
 VipsImage *
 vips_image_new_from_file( const char *filename )
 {
-	return( vips_image_new_mode( filename, "rd" ) ); 
+	return( vips_image_new_mode( filename, "r" ) ); 
 }
 
 /**
@@ -1514,19 +1762,23 @@ vips_image_new_from_memory( void *buffer,
 }
 
 /**
- * vips_image_new_array:
- * @xsize: image width
- * @ysize: image height
+ * vips_image_new_matrix:
+ * @width: image width
+ * @height: image height
  *
- * This convenience function makes an image which is an array: a one-band
+ * This convenience function makes an image which is a matrix: a one-band
  * VIPS_FORMAT_DOUBLE image held in memory.
  *
- * Use VIPS_IMAGE_ADDR() to address pixels in the image.
+ * Use VIPS_IMAGE_ADDR(), or VIPS_MATRIX() to address pixels in the image.
+ *
+ * Use vips_image_set_double() to set "scale" and "offset", if required. 
+ *
+ * See also: vips_image_new_matrixv()
  * 
  * Returns: the new #VipsImage, or %NULL on error.
  */
 VipsImage *
-vips_image_new_array( int xsize, int ysize )
+vips_image_new_matrix( int width, int height )
 {
 	VipsImage *image;
 
@@ -1534,13 +1786,13 @@ vips_image_new_array( int xsize, int ysize )
 
 	image = VIPS_IMAGE( g_object_new( VIPS_TYPE_IMAGE, NULL ) );
 	g_object_set( image,
-		"filename", "vips_image_new_array",
+		"filename", "vips_image_new_matrix",
 		"mode", "t",
-		"width", xsize,
-		"height", ysize,
+		"width", width,
+		"height", height,
 		"bands", 1,
 		"format", VIPS_FORMAT_DOUBLE,
-		"interpretation", VIPS_INTERPRETATION_ARRAY,
+		"interpretation", VIPS_INTERPRETATION_MATRIX,
 		NULL );
 	if( vips_object_build( VIPS_OBJECT( image ) ) ) {
 		VIPS_UNREF( image );
@@ -1555,18 +1807,52 @@ vips_image_new_array( int xsize, int ysize )
 	return( image );
 }
 
+/**
+ * vips_image_new_matrixv:
+ * @width: image width
+ * @height: image height
+ * @...: matrix coefficients
+ *
+ * As vips_image_new_matrix(), but initialise the matrix from the argument
+ * list. After @height should be @width * @height double constants which are
+ * used to set the matrix elements. 
+ *
+ * See also: vips_image_new_matrix()
+ * 
+ * Returns: the new #VipsImage, or %NULL on error.
+ */
+VipsImage *
+vips_image_new_matrixv( int width, int height, ... )
+{
+	va_list ap;
+	VipsImage *matrix;
+	int x, y;
+
+	vips_check_init();
+
+	matrix = vips_image_new_matrix( width, height ); 
+
+	va_start( ap, height );
+	for( y = 0; y < height; y++ )
+		for( x = 0; x < width; x++ )
+			*VIPS_MATRIX( matrix, x, y ) = va_arg( ap, double );
+	va_end( ap );
+
+	return( matrix ); 
+}
+
 /**
  * vips_image_set_delete_on_close:
  * @image: image to set
  * @delete_on_close: format of file
  *
  * Sets the delete_on_close flag for the image. If this flag is set, when
- * @image is finalized the filename held in @image->filename at the time of
+ * @image is finalized, the filename held in @image->filename at the time of
  * this call is unlinked.
  *
  * This function is clearly extremely dangerous, use with great caution.
  *
- * See also: vips__temp_name(), vips_image_new_disc_temp().
+ * See also: vips__temp_name(), vips_image_new_temp_file().
  */
 void
 vips_image_set_delete_on_close( VipsImage *image, gboolean delete_on_close )
@@ -1581,7 +1867,7 @@ vips_image_set_delete_on_close( VipsImage *image, gboolean delete_on_close )
 }
 
 /**
- * vips_image_new_disc_temp:
+ * vips_image_new_temp_file:
  * @format: format of file
  *
  * Make a "w" disc #VipsImage which will be automatically unlinked when it is
@@ -1594,7 +1880,7 @@ vips_image_set_delete_on_close( VipsImage *image, gboolean delete_on_close )
  * Returns: the new #VipsImage, or %NULL on error.
  */
 VipsImage *
-vips_image_new_disc_temp( const char *format )
+vips_image_new_temp_file( const char *format )
 {
 	char *name;
 	VipsImage *image;
@@ -1621,6 +1907,13 @@ vips_image_write_gen( VipsRegion *or,
 	VipsRegion *ir = (VipsRegion *) seq;
 	VipsRect *r = &or->valid;
 
+	/*
+	printf( "vips_image_write_gen: %p "
+		"left = %d, top = %d, width = %d, height = %d\n",
+		or->im,
+		r->left, r->top, r->width, r->height ); 
+	 */
+
 	/* Copy with pointers.
 	 */
 	if( vips_region_prepare( ir, r ) ||
@@ -1651,13 +1944,11 @@ vips_image_write( VipsImage *image, VipsImage *out )
         vips_demand_hint( out, 
 		VIPS_DEMAND_STYLE_THINSTRIP, image, NULL );
 
-	/* If this will be a delayed calculation we need to keep @image 
-	 * around for as long as @out is about.
+	/* We generate from @image partially, so we need to keep it about as
+	 * long as @out is about. 
 	 */
-	if( vips_image_ispartial( image ) ) {
-		g_object_ref( image );
-		vips_object_local( out, image );
-	}
+	g_object_ref( image );
+	vips_object_local( out, image );
 
 	if( vips_image_generate( out,
 		vips_start_one, vips_image_write_gen, vips_stop_one, 
@@ -1796,8 +2087,6 @@ vips_image_write_prepare( VipsImage *image )
 		break;
 
 	case VIPS_IMAGE_SETBUF:
-		/* Allocate memory.
-		 */
 		if( !image->data && 
 			!(image->data = vips_tracked_malloc( 
 				VIPS_IMAGE_SIZEOF_IMAGE( image ))) ) 
@@ -1845,7 +2134,7 @@ vips_image_write_line( VipsImage *image, int ypos, VipsPel *linebuffer )
 			return( -1 );
 
 		/* Always clear kill before we start looping. See the 
-		 * call to vips_image_get_kill() below.
+		 * call to vips_image_iskilled() below.
 		 */
 		vips_image_set_kill( image, FALSE );
 		vips_image_write_prepare( image );
@@ -1879,7 +2168,7 @@ vips_image_write_line( VipsImage *image, int ypos, VipsPel *linebuffer )
 	/* Trigger evaluation callbacks for this image.
 	 */
 	vips_image_eval( image, ypos * image->Xsize );
-	if( vips_image_get_kill( image ) )
+	if( vips_image_iskilled( image ) )
 		return( -1 );
 
 	/* Is this the end of eval?
@@ -1893,24 +2182,44 @@ vips_image_write_line( VipsImage *image, int ypos, VipsPel *linebuffer )
 	return( 0 );
 }
 
-/* Rewind an output file.
+/* Rewind an output file. VIPS images only.
  */
 static int
 vips_image_rewind_output( VipsImage *image ) 
 {
+	int fd;
+
+	g_assert( image->dtype == VIPS_IMAGE_OPENOUT );
+
 #ifdef DEBUG_IO
 	printf( "vips_image_rewind_output: %s\n", image->filename );
 #endif/*DEBUG_IO*/
 
+	/* We want to keep the fd across rewind. 
+	 *
+	 * On Windows, we open temp files with _O_TEMPORARY. We mustn't close
+	 * the file since this will delete it. 
+	 *
+	 * We could open the file again to keep a reference to it alive, but
+	 * this is also problematic on Windows. 
+	 */
+	fd = image->fd;
+	image->fd = -1;
+
 	/* Free any resources the image holds and reset to a base
 	 * state.
 	 */
 	vips_object_rewind( VIPS_OBJECT( image ) );
 
-	/* And reopen .. recurse to get a mmaped image.
+	/* And reopen ... recurse to get a mmaped image. 
+	 *
+	 * We use "v" mode to get it opened as a vips image, byopassing the
+	 * file type checks. They will fail on Windows becasue you can't open
+	 * fds more than once.
 	 */
+	image->fd = fd;
 	g_object_set( image,
-		"mode", "rd",
+		"mode", "v",
 		NULL );
 	if( vips_object_build( VIPS_OBJECT( image ) ) ) {
 		vips_error( "VipsImage", 
@@ -1928,7 +2237,7 @@ vips_image_rewind_output( VipsImage *image )
 	 *
 	 * On Windows this will fail because the file is open and you can't
 	 * delete open files. However, on Windows we set O_TEMP, so the file
-	 * will be deleted anyway on exit.
+	 * will be deleted when the fd is finally closed.
 	 */
 	vips_image_delete( image );
 
@@ -2095,7 +2404,8 @@ vips__image_wio_output( VipsImage *image )
  * @image: image to make read-write
  *
  * Gets @image ready for an in-place operation, such as im_insertplace().
- * Operations like this both read and write with VIPS_IMAGE_ADDR().
+ * After calling this function you can both read and write the image with 
+ * VIPS_IMAGE_ADDR().
  *
  * See also: im_insertplace(), vips_image_wio_input().
  *
@@ -2323,6 +2633,36 @@ vips_band_format_isuint( VipsBandFormat format )
 	}
 }
 
+/**
+ * vips_band_format_is8bit:
+ * @format: format to test
+ *
+ * Return %TRUE if @format is uchar or schar.
+ */
+gboolean
+vips_band_format_is8bit( VipsBandFormat format )
+{
+	switch( format ) {
+	case VIPS_FORMAT_UCHAR:
+	case VIPS_FORMAT_CHAR:
+		return( TRUE );
+
+	case VIPS_FORMAT_USHORT:
+	case VIPS_FORMAT_SHORT:
+	case VIPS_FORMAT_UINT:
+	case VIPS_FORMAT_INT:
+	case VIPS_FORMAT_FLOAT:
+	case VIPS_FORMAT_DOUBLE:
+	case VIPS_FORMAT_COMPLEX:
+	case VIPS_FORMAT_DPCOMPLEX:
+		return( FALSE );
+
+	default:
+		g_assert( 0 );
+		return( -1 );
+	}
+}
+
 /**
  * vips_band_format_isfloat:
  * @format: format to test
