@@ -59,7 +59,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -188,7 +189,7 @@ vips_region_finalize( GObject *gobject )
 {
 #ifdef VIPS_DEBUG
 	VIPS_DEBUG_MSG( "vips_region_finalize: " );
-	vips_object_print( VIPS_OBJECT( gobject ) );
+	vips_object_print_name( VIPS_OBJECT( gobject ) );
 #endif /*VIPS_DEBUG*/
 
 	G_OBJECT_CLASS( vips_region_parent_class )->finalize( gobject );
@@ -263,7 +264,7 @@ vips_region_dispose( GObject *gobject )
 
 #ifdef VIPS_DEBUG
 	VIPS_DEBUG_MSG( "vips_region_dispose: " );
-	vips_object_print( VIPS_OBJECT( gobject ) );
+	vips_object_print_name( VIPS_OBJECT( gobject ) );
 #endif /*VIPS_DEBUG*/
 
 	vips_object_preclose( VIPS_OBJECT( gobject ) );
@@ -323,6 +324,9 @@ vips_region_summary( VipsObject *object, VipsBuf *buf )
 	vips_buf_appendf( buf, "width = %d, ", region->valid.width );
 	vips_buf_appendf( buf, "height = %d", region->valid.height );
 
+	if( region->buffer && region->buffer->buf )
+		vips_buf_appendf( buf, ", bytes = %zd", region->buffer->bsize );
+
 	VIPS_OBJECT_CLASS( vips_region_parent_class )->summary( object, buf );
 }
 
@@ -369,7 +373,8 @@ vips__region_take_ownership( VipsRegion *region )
 		 * Not sure if this will ever happen: if it does, we'll 
 		 * need to dup the buffer.
 		 */
-		g_assert( !region->buffer || region->buffer->ref_count == 1 );
+		g_assert( !region->buffer || 
+			region->buffer->ref_count == 1 );
 
 		region->thread = g_thread_self();
 	}
@@ -866,8 +871,9 @@ vips_region_paint( VipsRegion *reg, VipsRect *r, int value )
 	vips_rect_intersectrect( r, &reg->valid, &ovl );
 	if( !vips_rect_isempty( &ovl ) ) {
 		VipsPel *q = VIPS_REGION_ADDR( reg, ovl.left, ovl.top );
-		int wd = ovl.width * VIPS_IMAGE_SIZEOF_PEL( reg->im );
-		int ls = VIPS_REGION_LSKIP( reg );
+		size_t wd = ovl.width * VIPS_IMAGE_SIZEOF_PEL( reg->im );
+		size_t ls = VIPS_REGION_LSKIP( reg );
+
 		int y;
 
 		for( y = 0; y < ovl.height; y++ ) {
@@ -1051,7 +1057,7 @@ vips_region_prepare( VipsRegion *reg, VipsRect *r )
 
 	vips__region_check_ownership( reg );
 
-	if( vips_image_get_kill( im ) )
+	if( vips_image_iskilled( im ) )
 		return( -1 );
 
 	/* We use save for sanity checking valid: we test at the end that the
@@ -1185,7 +1191,7 @@ vips_region_prepare_to( VipsRegion *reg,
 	VipsRect clipped2;
 	VipsRect final;
 
-	if( vips_image_get_kill( im ) )
+	if( vips_image_iskilled( im ) )
 		return( -1 );
 
 	/* Sanity check.
