@@ -32,7 +32,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -59,7 +60,7 @@
 
 /**
  * SECTION: interpolate
- * @short_description: various interpolators: nearest, bilinear, bicubic, and
+ * @short_description: various interpolators: nearest, bilinear, and
  * some non-linear
  * @stability: Stable
  * @include: vips/vips.h
@@ -78,7 +79,7 @@ G_DEFINE_ABSTRACT_TYPE( VipsInterpolate, vips_interpolate, VIPS_TYPE_OBJECT );
  * @y: interpolate value at this position
  *
  * An interpolation function. It should read source pixels from @in with
- * IM_REGION_ADDR(), it can look left and up from (x, y) by @window_offset
+ * VIPS_REGION_ADDR(), it can look left and up from (x, y) by @window_offset
  * pixels and it can access pixels in a window of size @window_size.
  *
  * The interpolated value should be written to the pixel pointed to by @out.
@@ -127,7 +128,7 @@ static void
 vips_interpolate_finalize( GObject *gobject )
 {
 	printf( "vips_interpolate_finalize: " );
-	vips_object_print( VIPS_OBJECT( gobject ) );
+	vips_object_print_name( VIPS_OBJECT( gobject ) );
 
 	G_OBJECT_CLASS( vips_interpolate_parent_class )->finalize( gobject );
 }
@@ -159,7 +160,7 @@ vips_interpolate_real_get_window_offset( VipsInterpolate *interpolate )
 
 		/* Don't go -ve, of course, for window_size 1.
 		 */
-		return( IM_MAX( 0, window_size / 2 - 1 ) );
+		return( VIPS_MAX( 0, window_size / 2 - 1 ) );
 	}
 }
 
@@ -191,7 +192,7 @@ vips_interpolate_init( VipsInterpolate *interpolate )
 {
 #ifdef DEBUG
 	printf( "vips_interpolate_init: " );
-	vips_object_print( VIPS_OBJECT( interpolate ) );
+	vips_object_print_name( VIPS_OBJECT( interpolate ) );
 #endif /*DEBUG*/
 }
 
@@ -211,7 +212,7 @@ vips_interpolate_init( VipsInterpolate *interpolate )
  */
 void
 vips_interpolate( VipsInterpolate *interpolate,
-	void *out, REGION *in, double x, double y )
+	void *out, VipsRegion *in, double x, double y )
 {
 	VipsInterpolateClass *class = VIPS_INTERPOLATE_GET_CLASS( interpolate );
 
@@ -329,19 +330,14 @@ G_DEFINE_TYPE( VipsInterpolateNearest, vips_interpolate_nearest,
 
 static void
 vips_interpolate_nearest_interpolate( VipsInterpolate *interpolate,
-	void *out, REGION *in, double x, double y )
+	void *out, VipsRegion *in, double x, double y )
 {
-	/* Pel size and line size.
-	 */
-	const int ps = IM_IMAGE_SIZEOF_PEL( in->im );
+	const int ps = VIPS_IMAGE_SIZEOF_PEL( in->im );
 
-	/* Top left corner we interpolate from. We know x/y are always
-	 * positive, so we can just (int) them.
-	 */
 	const int xi = (int) x;
 	const int yi = (int) y;
 
-	const VipsPel *p = IM_REGION_ADDR( in, xi, yi );
+	const VipsPel *p = VIPS_REGION_ADDR( in, xi, yi );
 	VipsPel *q = (VipsPel *) out;
 
 	int z;
@@ -369,7 +365,7 @@ vips_interpolate_nearest_init( VipsInterpolateNearest *nearest )
 {
 #ifdef DEBUG
 	printf( "vips_interpolate_nearest_init: " );
-	vips_object_print( VIPS_OBJECT( nearest ) );
+	vips_object_print_name( VIPS_OBJECT( nearest ) );
 #endif /*DEBUG*/
 }
 
@@ -484,14 +480,14 @@ G_DEFINE_TYPE( VipsInterpolateBilinear, vips_interpolate_bilinear,
  */
 #define SWITCH_INTERPOLATE( FMT, INT, FLOAT ) { \
 	switch( (FMT) ) { \
-	case IM_BANDFMT_UCHAR:	INT( unsigned char ); break; \
-	case IM_BANDFMT_CHAR: 	INT( char ); break;  \
-	case IM_BANDFMT_USHORT: INT( unsigned short ); break;  \
-	case IM_BANDFMT_SHORT: 	INT( short ); break;  \
-	case IM_BANDFMT_UINT: 	FLOAT( unsigned int ); break;  \
-	case IM_BANDFMT_INT: 	FLOAT( int );  break;  \
-	case IM_BANDFMT_FLOAT: 	FLOAT( float ); break;  \
-	case IM_BANDFMT_DOUBLE:	FLOAT( double ); break;  \
+	case VIPS_FORMAT_UCHAR:	INT( unsigned char ); break; \
+	case VIPS_FORMAT_CHAR: 	INT( char ); break;  \
+	case VIPS_FORMAT_USHORT:INT( unsigned short ); break;  \
+	case VIPS_FORMAT_SHORT: INT( short ); break;  \
+	case VIPS_FORMAT_UINT: 	FLOAT( unsigned int ); break;  \
+	case VIPS_FORMAT_INT: 	FLOAT( int );  break;  \
+	case VIPS_FORMAT_FLOAT: FLOAT( float ); break;  \
+	case VIPS_FORMAT_DOUBLE:FLOAT( double ); break;  \
 	default: \
 		g_assert( FALSE ); \
 	} \
@@ -499,18 +495,18 @@ G_DEFINE_TYPE( VipsInterpolateBilinear, vips_interpolate_bilinear,
 
 static void
 vips_interpolate_bilinear_interpolate( VipsInterpolate *interpolate,
-	void *out, REGION *in, double x, double y )
+	void *out, VipsRegion *in, double x, double y )
 {
 	/* Pel size and line size.
 	 */
-	const int ps = IM_IMAGE_SIZEOF_PEL( in->im );
-	const int ls = IM_REGION_LSKIP( in );
+	const int ps = VIPS_IMAGE_SIZEOF_PEL( in->im );
+	const int ls = VIPS_REGION_LSKIP( in );
 	const int b = in->im->Bands;
 
 	const int ix = (int) x;
 	const int iy = (int) y;
 
-	const VipsPel *p1 = IM_REGION_ADDR( in, ix, iy );
+	const VipsPel *p1 = VIPS_REGION_ADDR( in, ix, iy );
 	const VipsPel *p2 = p1 + ps;
 	const VipsPel *p3 = p1 + ls;
 	const VipsPel *p4 = p3 + ps;
@@ -540,7 +536,7 @@ vips_interpolate_bilinear_init( VipsInterpolateBilinear *bilinear )
 {
 #ifdef DEBUG
 	printf( "vips_interpolate_bilinear_init: " );
-	vips_object_print( VIPS_OBJECT( bilinear ) );
+	vips_object_print_name( VIPS_OBJECT( bilinear ) );
 #endif /*DEBUG*/
 
 }
