@@ -16,7 +16,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -367,7 +368,7 @@ double VImage::meta_get_double( const char *field )
 const char *VImage::meta_get_string( const char *field ) 
 	throw( VError )
 {
-	char *result;
+	const char *result;
 
 	if( im_meta_get_string( _ref->im, field, &result ) )
 		verror();
@@ -484,6 +485,8 @@ Vargv::~Vargv()
 					io->vec = NULL;
 				}
 			}
+			else if( strcmp( ty->type, IM_TYPE_INTERPOLATE ) == 0 )
+				g_object_unref( base[i] );
 		}
 	}
 
